import wx
import garlicsim.data_structures


class StateCreationDialog(wx.Dialog):
    # This is where you define a dialog for creating a root state.
    #
    # When someone loads your simpack into `garlicsim_wx` and starts a new
    # simulation, the first thing he sees is the state creation dialog. The
    # state creation dialog is a convenient way to create the first state in
    # your program.
    #
    # This dialog should display some kind of form for the user to fill out.
    # This will usually ask the user question about how they want their initial
    # state to be. For example, for a Life simpack the dialog will ask what
    # should be the width and height of the new state.
    
    def __init__(self, frame):
        wx.Dialog.__init__(self, frame, title="Creating a root state")
        
        self.frame = frame

        # ...

        
    def start(self):
        # The `start` method will be used by `garlicsim_wx` after the dialog has
        # been created to make the dialog appear on the screen, and to create a
        # state object from the data that the dialog has collected from the
        # user. Returns a state, or None if the user hit "cancel" or something.
        if self.ShowModal() == wx.ID_OK:
            # state = ...
            pass
        else:
            state = None
        self.Destroy()
        return state
        


    
    


import time
import sys
import multiprocessing

class Stopper(object):
    pass

def dump(queue):
    '''
    Empties all pending items in a queue and returns them in a list.
    
    Use only when no other processes/threads are reading from the queue.
    '''
    result = []

    # START DEBUG CODE
    initial_size = queue.qsize()
    print(("Queue has %s items initially." % initial_size))
    #  END  DEBUG CODE

    queue.put(Stopper)
    #queue.put(SecondStopper)
    
    for thing in iter(queue.get, Stopper): # todo sentinel=
        result.append(thing)
    
    # START DEBUG CODE
    current_size = queue.qsize()
    total_size = current_size + len(result)
    print("Dumping complete:")
    if current_size == initial_size:
        print("No items were added to the queue.")
    else:
        print(("%s items were added to the queue." % \
              (total_size - initial_size)))
    print(("Extracted %s items from the queue, queue has %s items left" \
    % (len(result), current_size)))
    #  END  DEBUG CODE
            
    return result



class Process(multiprocessing.Process):
    def __init__(self):
        multiprocessing.Process.__init__(self)
        self.q = multiprocessing.Queue()
    def run(self):
        for i in range(100):
            self.q.put([list(range(20)) for j in range(10)])
            sys.stdout.write('Put item %s.\n' % i)
            sys.stdout.flush()



if __name__ == '__main__':
    p = Process()
    p.start()
    # p.join()
    time.sleep(10)
    l = dump(p.q)



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''A package for doing a binary search in a sequence.'''

from .roundings import (Rounding, LOW, LOW_IF_BOTH, LOW_OTHERWISE_HIGH, HIGH,
                        HIGH_IF_BOTH, HIGH_OTHERWISE_LOW, EXACT, CLOSEST,
                        CLOSEST_IF_BOTH, BOTH)
from .functions import (binary_search, binary_search_by_index,
                        make_both_data_into_preferred_rounding)
from .binary_search_profile import BinarySearchProfile


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This module defines string-related tools.'''


import re


def camelcase_to_underscore(s):
    '''
    Convert a string from camelcase to underscore.
    
    Example: camelcase_to_underscore('HelloWorld') == 'hello_world'
    '''
    return re.sub('(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))', '_\\1', s).lower().strip('_')


def camelcase_to_spacecase(s):
    '''
    Convert a string from camelcase to spacecase.
    
    Example: camelcase_to_underscore('HelloWorld') == 'Hello world'
    '''
    if s == '': return s
    character_process = lambda c: (' ' + c.lower()) if c.isupper() else c
    return s[0] + ''.join(character_process(c) for c in s[1:])

"""
def underscore_to_camelcase(s):
    raise NotImplementedError
"""




# from .widgets import state_viewer
# from .widgets import state_creation_dialog


# BIG_WORKSPACE_WIDGETS = state_viewer.StateViewer

# Widgets to show in the middle of the frame. These must be instances of
# garlicsim_wx.widgets.workspace_widget.
#
# This is where you usually put your main widget that displays your states.


# SMALL_WORKSPACE_WIDGETS = []

# (05.15.2010 - Still not implemented, sorry.)
#
# Small widgets to show the in frame. These must be instances of
# garlicsim_wx.widgets.workspace_widget.
#
# This is where you usually put small tools.


# SEEK_BAR_GRAPHS = []

# (05.15.2010 - Still not implemented, sorry.)
#
# List of scalar functions that should be shown as graphs in the seek bar.
#
# These may be either scalar state functions or scalar history functions.


# STATE_CREATION_DIALOG = widgets.state_creation_dialog
#
# Dialog for creating a root state.


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines objects for working with simulations that crunch asynchronically.

(Asynchronously means in a separate thread/process.)

The most important class defined here is Project, and it is the only class that
the user needs to interact with. It employs all the other classes.
'''

from .obsolete_cruncher_error import ObsoleteCruncherError
from .history_browser import HistoryBrowser
from .crunching_profile import CrunchingProfile
from . import crunchers
from .project import Project
from .job import Job
from .crunching_manager import CrunchingManager

__all__ = ['Project', 'HistoryBrowser', 'CrunchingManager', 'Job',
           'crunchers', 'CrunchingProfile', 'ObsoleteCruncherError']




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines settings.'''

class Setting(object):
    '''
    A setting.
    
    These are used as classes without insantiating.
    '''

class DeterminismSetting(Setting):
    '''
    A setting of determinism.
    
    When GarlicSim knows that a certain step profile is deterministic, it can
    help it analyze the simulation. For example, it lets GarlicSim detect when
    the simulation has reached a constant/repetitive state.
    '''

class UNDETERMINISTIC(DeterminismSetting):
    '''Completely not deterministic -- has a random element.'''

class SUPPOSEDLY_DETERMINISTIC(DeterminismSetting):
    '''
    Means deterministic in principle, but not absolutely.
    
    For example, in some simpacks rounding errors may make states that should
    otherwise be equal not be equal.
    '''
    # todo: possibly rename, use thesaurus

class DETERMINISTIC(DeterminismSetting):
    '''
    Absolutely deterministic.
    
    There is no random or fuzzy element in the step function, and given
    identical input states it is guaranteed to return identical output states.
    '''




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CruncherProcess class.

See its documentation for more information.
'''

import multiprocessing

import copy
import queue
import sys
import os

try: import garlicsim.general_misc.process_priority
except Exception: pass

import garlicsim
from garlicsim.asynchronous_crunching import \
     CrunchingProfile, ObsoleteCruncherError

__all__ = ["CruncherProcess"]

class CruncherProcess(multiprocessing.Process):
    '''
    CruncherProcess is a type of cruncher the works from a process.
    
    A cruncher is a worker which crunches the simulation. It receives a state
    from the main program, and then it repeatedly applies the step function of
    the simulation to produce more states. Those states are then put in the
    cruncher's work_queue. They are then taken by the main program when
    Project.sync_crunchers is called, and put into the tree.
        
    Read more about crunchers in the documentation of the crunchers package.
    
    The advantage of CruncherProcess over CruncherThread is that
    CruncherProcess is able to run on a different core of the processor
    in the machine, thus using the full power of the processor.
    '''
    # todo: probably encapsulate Process, because it pickles all __init__
    # arguments and we want unified cruncher __init__.
    
    def __init__(self, initial_state, step_generator, crunching_profile):
        
        multiprocessing.Process.__init__(self)
        
        self.step_generator = step_generator
        self.initial_state = initial_state
        self.last_clock = self.initial_state.clock
        self.crunching_profile = crunching_profile
        
        self.daemon = True

        self.work_queue = multiprocessing.Queue()
        '''
        Queue for putting completed work to be picked up by the main thread.
        
        In this queue the cruncher will put the states that it produces, in
        chronological order. If the cruncher is being given a new crunching
        profile which has a new and different step profile, the cruncher
        will put the new step profile in this queue in order to signal that
        from that point on, all states were crunched with that step profile.
        '''
        
        self.order_queue = multiprocessing.Queue()
        '''Queue for receiving instructions from the main thread.'''

    def set_low_priority(self):
        '''Set a low priority for this process.'''
        
        try:
            sys.getwindowsversion()
        except Exception:
            is_windows = False
        else:
            is_windows = True

        if is_windows:
            try:
                garlicsim.general_misc.process_priority.set_process_priority(0)
            except Exception:
                pass
        else:
            try:
                os.nice(1)
            except Exception:
                pass

    def run(self):
        '''
        Internal method.
        
        This is called when the cruncher is started. It just calls the
        main_loop method in a try clause, excepting ObsoleteCruncherError;
        That exception means that the cruncher has been retired in the middle
        of its job, so it is propagated up to this level, where it causes the
        cruncher to terminate.
        '''
        try:
            self.main_loop()
        except ObsoleteCruncherError:
            return

    def main_loop(self):
        '''
        The main loop of the cruncher.
        
        Crunches the simulations repeatedly until the crunching profile is
        satisfied or a 'retire' order is received.
        '''
        self.set_low_priority()
        
        state = self.initial_state
        
        self.step_profile = self.crunching_profile.step_profile
        
        self.iterator = self.step_generator(self.initial_state,
                                            self.step_profile)
        
        order = None
        
        try:
            for state in self.iterator:
                self.work_queue.put(state)
                self.check_crunching_profile(state)
                order = self.get_order()
                if order:
                    self.process_order(order) 
        except garlicsim.misc.WorldEnd:
            self.work_queue.put(garlicsim.asynchronous_crunching.misc.EndMarker())
        
    def check_crunching_profile(self, state):
        '''
        Check if the cruncher crunched enough states. If so retire.
        
        The crunching manager specifies how much the cruncher should crunch.
        We consult with it to check if the cruncher has finished, and if it did
        we retire the cruncher.
        '''
        if self.crunching_profile.state_satisfies(state):
            raise ObsoleteCruncherError
    
        
    def get_order(self):
        '''
        Attempt to read an order from the order_queue, if one has been sent.
        
        Returns the order.
        '''
        try:
            return self.order_queue.get(block=False)
        except queue.Empty:
            return None
    
        
    def process_order(self, order):
        '''Process an order receieved from order_queue.'''
        
        if order == "retire":
            raise ObsoleteCruncherError
        
        elif isinstance(order, CrunchingProfile):
            self.process_crunching_profile_order(order)
            
            
            
    def process_crunching_profile_order(self, order):
        '''Process an order to update the crunching profile.'''
        if self.crunching_profile.step_profile != order.step_profile:
            self.work_queue.put(order.step_profile)
        self.crunching_profile = order
        self.step_profile = order.step_profile            
        self.iterator.set_step_profile(self.step_profile)
        
            
    def retire(self):
        '''
        Retire the cruncher. Process-safe.
        
        Causes it to shut down as soon as it receives the order.
        '''
        self.order_queue.put("retire")
        
        
    def update_crunching_profile(self, profile):
        '''Update the cruncher's crunching profile. Process-safe.'''
        self.order_queue.put(profile)
        
        
        
        
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A module that defines the Tree class and the related TreeError exception.

See their documentation for more information.
'''

import copy

from garlicsim.general_misc import misc_tools

import garlicsim.misc
from garlicsim.misc import GarlicSimException

# `from block import Block` in the bottom of the file.
# `from node import Node` in the bottom of the file.


__all__ = ["Tree", "TreeError"]


class TreeError(GarlicSimException):
    '''Tree-related exception.'''

class Tree(object):
    '''
    A time tree, (generalization of timeline,) of the simulation.

    A tree is used within a project to organize everything that is happenning in
    the simulation.
    
    Often, when doing a simulation, the tree will be a degenerate tree, i.e. a
    straight, long succession of nodes with no more than one child each. The
    meaning of one node in the tree being another node's child is that the
    child node comes after the parent node in the timeline.
    
    Trees are useful, because they give you the ability to "split" or "fork"
    the simulation at any node you wish, allowing you to explore and analyze
    different scenarios in parallel in the same simulation.

    Each node in the tree may have a parent, or may not, in which case it will
    also be called a root and be a member of `.roots`.
    '''
    def __init__(self):
        
        self.nodes = []
        '''List of nodes that belong to the tree.'''
        
        self.roots = []
        '''List of roots (parentless nodes) of the tree.'''
        
        self.lock = garlicsim.general_misc.read_write_lock.ReadWriteLock()
        '''
        A read-write lock that guards access to the tree.
        
        We need such a thing because some simulations are history-dependent and
        require reading from the tree in the same time that sync_crunchers could
        potentially be writing to it.
        '''

        
    def fork_to_edit(self, template_node):
        '''
        "Duplicate" the node, marking the new one as touched.
        
        The new node will have the same parent as `template_node`. The state of
        the new node is usually modified by the user after it is created, and
        after that the node is finalized and used in simulation.
        
        This is useful when you want to make some changes in the world state and
        see what they will cause in the simulation.
        
        Returns the node.
        '''
        new_state = copy.deepcopy(
            template_node.state,
            garlicsim.misc.persistent.DontCopyPersistent()
        )

        parent = template_node.parent
        new_step_profile = copy.copy(template_node.step_profile)
        new_node = self.add_state(new_state, parent,
                                  step_profile=new_step_profile,
                                  template_node=template_node)
        new_node.still_in_editing = True
        return new_node


    def add_state(self, state, parent=None, step_profile=None,
                  template_node=None):
        '''
        Wrap state in node and add to tree.
        
        Returns the node.
        '''
        touched = (parent is None) or (template_node is not None)
        
        my_node = Node(
            self,
            state,
            step_profile=copy.copy(step_profile),
            touched=touched
        )
        
        self.__add_node(my_node, parent, template_node)
        return my_node


    def __add_node(self, node, parent=None, template_node=None):
        '''
        Add a node to the tree.
        
        It may be a natural node or a touched node. If it's a natural node you
        may not specify a template_node.
        
        Returns the node.
        '''
        if template_node is not None:
            if parent != template_node.parent:
                raise TreeError('''Parent you specified and parent of \
template_node aren't the same!''')
            if not node.touched:
                raise TreeError('''You tried adding an untouched state to a \
tree while specifying a template_node.''')
            template_node.derived_nodes.append(node)
            

        self.nodes.append(node)

        if parent:
            if not hasattr(node.state, 'clock'):
                node.state.clock = parent.state.clock + 1

            node.parent = parent
            parent.children.append(node)
            
            if parent.block:
                
                if len(parent.children) == 1:
                    
                    if (not node.touched) and \
                       (parent.step_profile == node.step_profile):
                        
                        parent.block.append_node(node)
                        
                else: # parent.children > 1

                    if not (parent is parent.block[-1]):
                        
                        parent.block.split(parent)
                        
            else: # parent.block is None
                
                if (not node.touched) and \
                   (not parent.touched) and \
                   (len(parent.children)==1) and \
                   (parent.step_profile == node.step_profile):
                    
                    Block([parent, node])
                
                        
        else: # parent is None
            if not hasattr(node.state, "clock"):
                node.state.clock = 0
            self.roots.append(node)
            return node

    
    def make_end(self, node, step_profile):
        '''
        Create an end after the specified node.
        
        Must specify a step profile with which this end was reached.
        '''
        end = End(self, node, step_profile)
        return end
    

    def all_possible_paths(self):
        '''Return all the possible paths this tree may entertain.'''
        result = []
        for root in self.roots:
            result += root.all_possible_paths()
        return result
    
    
    def delete_node_selection(self, node_selection):
        '''
        Delete a node selection from the tree.
        
        Any nodes that will be orphaned by this deletion will become roots.
        '''
                
        stitch = False
        # todo: this is supposed to be an argument allowing the children to be
        # stitched to the new parent, but I'm currently forcing it to be false
        # because I haven't decided yet how I will handle stitching.
        
        node_selection.compact()
        for node_range in node_selection.ranges:
            self.delete_node_range(node_range) #, stitch=stitch)

            
    def delete_node_range(self, node_range):
        '''
        Delete a node range from the tree.
        
        Any nodes that will be orphaned by this deletion will become roots.
        '''
        
        stitch = False
        # todo: this is supposed to be an argument allowing the children to be
        # stitched to the new parent, but I'm currently forcing it to be false
        # because I haven't decided yet how I will handle stitching.
        
        start_node = node_range.start if isinstance(node_range.start, Node) \
                     else node_range.start[0]
        
        end_node = node_range.end if isinstance(node_range.end, Node) \
                     else node_range.end[-1]
        
        if start_node in self.roots:
            self.roots.remove(start_node)
                        
        big_parent = start_node.parent
        if big_parent is not None:
            big_parent.children.remove(start_node)
        
        outside_children = node_range.get_outside_children()
            
        for node in node_range:
            self.nodes.remove(node)

        current_block = None
        last_block_change = None
        for node in node_range:
            if node.block is not current_block:
                if current_block is not None:
                    del current_block[current_block.index(last_block_change) :
                                      current_block.index(node.parent)]
                current_block = node.block
                last_block_change = node
                
        if current_block is not None:
            del current_block[current_block.index(last_block_change) :
                              current_block.index(end_node)]
                    
        parent_to_use = big_parent if (stitch is True) else None
        for node in outside_children:
            node.parent = parent_to_use
            if parent_to_use is None:
                self.roots.append(node)
        
    
    
    """ todo: In construction:
    def move_node_range(self, node_range):
        pass
    
    def copy_node_range(self, node_range, start=None, end=None):
        pass
    """

    
    def __repr__(self):
        '''
        Get a string representation of the tree.
        
        Example output:        
        <garlicsim.data_structures.Tree with 1 roots, 233 nodes and 3 possible
        paths at 0x1f6ae70>
        '''
        return '<%s with %s roots, %s nodes and %s possible paths at %s>' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                   ),
                   len(self.roots),
                   len(self.nodes),
                   len(self.all_possible_paths()),
                   hex(id(self))
               )
    
    
    def __getstate__(self):
        my_dict = dict(self.__dict__)
        del my_dict['lock']
        return my_dict
    
    
    def __setstate__(self, pickled_tree):
        self.__init__()
        self.__dict__.update(pickled_tree)
        
        
    
from .node import Node
from .block import Block
from .end import End




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Module for doing a binary search in a sequence.'''

# Todo: wrap all things in tuples?
# 
# todo: add option to specify cmp.
# 
# todo: i think `binary_search_by_index` should have the core logic, and the
# other one will use it. I think this will save many sequence accesses, and some
# sequences can be expensive.
#
# todo: decide already if we return only tuples or only lists. Probably tuples.
#
# todo: ensure there are no `if variable` checks where we're thinking of None
# but the variable might be False

from .roundings import (Rounding, roundings, LOW, LOW_IF_BOTH,
                        LOW_OTHERWISE_HIGH, HIGH, HIGH_IF_BOTH,
                        HIGH_OTHERWISE_LOW, EXACT, CLOSEST, CLOSEST_IF_BOTH,
                        BOTH)

def binary_search_by_index(sequence, function, value, rounding=CLOSEST):
    '''
    Do a binary search, returning answer as index number.
    
    Similiar to binary_search (refer to its documentation for more info). The
    difference is that instead of returning a result in terms of sequence items,
    it returns the indexes of these items in the sequence.
    
    For documentation of rounding options, check `binary_search.roundings`.
    ''' 
    if function is None:
        function = lambda x: x
    my_range = range(len(sequence))
    fixed_function = lambda index: function(sequence[index])
    result = binary_search(my_range, fixed_function, value, rounding)
    return result


def binary_search(sequence, function, value, rounding=CLOSEST):
    '''
    Do a binary search through a sequence.
    
    It is assumed that `function` is a monotonic rising function on `sequence`.
    
    For all rounding options, a return value of None is returned if no matching
    item is found. (In the case of rounding=BOTH, either of the items in the
    tuple may be None)
    
    Note: This function uses None to express its inability to find any matches;
    Therefore, you better not use it on sequences in which None is a possible
    item.
    
    For documentation of rounding options, check `binary_search.roundings`.
    '''
    
    # todo: can break this into __binary_search_with_both_rounding

    # todo: i think this should be changed to return tuples
    
    assert issubclass(rounding, Rounding)
    
    if function is None:
        function = lambda x: x
    
    if not sequence:
        if rounding is BOTH:
            return (None, None)
        else:
            return None
    
    get = lambda number: function(sequence[number])

    low = 0
    high = len(sequence) - 1

    low_value, high_value = get(low), get(high)

    
    if low_value >= value:

        if rounding is BOTH:
            return [None if low_value > value else sequence[low], sequence[low]]
        
        if rounding in (HIGH, HIGH_OTHERWISE_LOW, CLOSEST) or \
           (low_value==value and rounding is EXACT):
            return sequence[low]
        
        else:
            # rounding in (LOW*, *_IF_BOTH) or (rounding is EXACT and
            # low_value!=value)
            return None

        
    if high_value <= value:

        if rounding is BOTH:
            return [
                sequence[high],
                None if high_value < value else sequence[high]
            ]
        
        if rounding in (LOW, LOW_OTHERWISE_HIGH, CLOSEST) or \
           (low_value==value and rounding is EXACT):
            return sequence[high]
        
        else:
            # rounding in (HIGH*, *_IF_BOTH) or (rounding is EXACT and
            # low_value!=value)
            return None
        

    # Now we know the value is somewhere inside the sequence.
    
    
    while high - low > 1:
        medium = (low + high) // 2
        medium_value = get(medium)
        if medium_value > value:
            high = medium; high_value = medium_value
            continue
        if medium_value < value:
            low = medium; low_value = medium_value
            continue
        if medium_value == value:
            if rounding is BOTH:
                return (sequence[medium], sequence[medium])
            after_medium = medium + 1;
            after_medium_value = get(after_medium)
            if after_medium_value == value:
                low = medium; low_value = medium_value
                high = after_medium; high_value = after_medium_value
                break
            else: # get(medium+1) > value
                high = medium; high_value = medium_value
                low = medium - 1; low_value = get(low)
                break
    
    both = (sequence[low], sequence[high])
    
    return make_both_data_into_preferred_rounding(both, function,\
                                                  value, rounding)

def make_both_data_into_preferred_rounding(both, function, value, rounding):
    '''
    Convert results gotten using `BOTH` to a different rounding option.
    
    This function takes the return value from `binary_search` (or other such
    functions) with rounding=BOTH as the parameter `both`. It then gives the
    data with a different rounding, specified with the parameter `rounding`.
    '''
    # todo optimize and organize: break to individual functions, put in
    # BinarySearchProfile
    if rounding is BOTH:
        return both
    
    elif rounding is LOW:
        return both[0]
    
    elif rounding is LOW_IF_BOTH:
        return both[0] if both[1] is not None else None
    
    elif rounding is LOW_OTHERWISE_HIGH:
        return both[0] if both[0] is not None else both[1]
    
    elif rounding is HIGH:
        return both[1]
    
    elif rounding is HIGH_IF_BOTH:
        return both[1] if both[0] is not None else None
    
    elif rounding is HIGH_OTHERWISE_LOW:
        return both[1] if both[1] is not None else both[0]
    
    elif rounding is EXACT:
        results = [state for state in both if
                   (state is not None and function(state) == value)
                   ]
        return results[0] if results else None
    
    elif rounding in (CLOSEST, CLOSEST_IF_BOTH):
        if rounding is CLOSEST_IF_BOTH:
            if None in both:
                return None
        if both[0] is None: return both[1]
        if both[1] is None: return both[0]
        distances = [abs(function(state)-value) for state in both]
        if distances[0] <= distances[1]:
            return both[0]
        else:
            return both[1]
        
        


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the HistoryBrowser class.

See its documentation for more info.
'''



import threading

import garlicsim.general_misc.binary_search as binary_search
import garlicsim.general_misc.queue_tools as queue_tools
import garlicsim.general_misc.third_party.decorator

import garlicsim.misc
from .obsolete_cruncher_error import ObsoleteCruncherError

__all__ = ["HistoryBrowser"]

@garlicsim.general_misc.third_party.decorator.decorator
def with_self(method, *args, **kwargs):
    '''Decorator for using the history browser as a context manager.'''
    self = args[0]
    with self:
        return method(*args, **kwargs)

class HistoryBrowser(garlicsim.misc.BaseHistoryBrowser):
    '''
    A device for requesting information about the history of the simulation.
    
    A HistoryBrowser is a device for requesting information about the history of
    the simulation. It is intended to be used by CruncherThread in simulations
    that are history-dependent.
    
    With a HistoryBrowser one can request states from the simulation's timeline.
    States can be requested by clock time or position in the timeline or by
    other measures; See documentation for this class's methods.
    
    Since we do not know whether the states we request have been implemented in
    the tree already, or they are still in the work_queue, it's the job of the
    HistoryBrowser to find that out. This is done transperantly for the user.
    
    When using a HistoryBroswer, the lock of the project's tree is acquired for
    reading. That acquiring action can also be invoked by using HistoryBrowser
    as a context manager.
    '''
    
    def __init__(self, cruncher):
        self.cruncher = cruncher
        self.project = cruncher.project
        self.tree = self.project.tree
        self.tree_lock = self.project.tree.lock
    
    def __enter__(self, *args, **kwargs):
        '''Acquire the lock of the project's tree for reading.'''
        self.tree_lock.acquireRead()
    
    def __exit__(self, *args, **kwargs):
        '''Release the project's tree_lock.'''
        self.tree_lock.release()
     
    @with_self
    def get_last_state(self):
        '''Get the last state in the timeline. Identical to __getitem__(-1).'''
        return self[-1]
    
    @with_self
    def __getitem__(self, index):
        '''Get a state by its position in the timeline.'''
        assert isinstance(index, int)
        if index < 0:
            return self.__get_item_negative(index)
        else: # index >= 0
            return self.__get_item_positive(index)
    
    @with_self
    def __get_item_negative(self, index):
        '''
        Get a state by its position in the timeline. Negative indices only.
        '''
        try:
            return self.__get_item_from_queue(index)
        except IndexError:
            # The requested state is in the tree
            queue_size = self.cruncher.work_queue.qsize()
            new_index = index + queue_size
            our_node = self.__get_our_node()
            path = our_node.make_containing_path()
            result_node = path.__getitem__(new_index, end=our_node)
            return result_node.state
            
    
    @with_self
    def __get_item_positive(self, index):
        '''
        Get a state by its position in the timeline. Positive indices only.
        '''
        our_node = self.__get_our_node()
        path = our_node.make_containing_path()
        try:
            result_node = path.__getitem__(index, end=our_node)
            return result_node.state
        
        except IndexError:
            path_length = path.__len__(end=our_node)
            # todo: Probably inefficient: We're plowing through the path again.
            new_index = index - path_length
            try:
                return self.__get_item_from_queue(new_index)
            except IndexError:
                queue_length = self.cruncher.work_queue.qsize()
                timeline_length = queue_length + path_length
                message = "You asked for node number " + str(index) + \
                          " while the timeline has only " + timeline_length + \
                          " states, comprised by " + path_length + \
                          " states in the tree and " + queue_length + \
                          " states in the queue."
                raise IndexError(message)
        
    @with_self
    def __get_item_from_queue(self, index):
        '''
        Obtain an item by index number from the work_queue of our cruncher.
        '''
        item = queue_tools.get_item(self.cruncher.work_queue, index)
        return item
        
    @with_self
    def get_state_by_monotonic_function(self, function, value,
                                        rounding=binary_search.CLOSEST):
        '''
        Get a state by specifying a measure function and a desired value.
        
        The function must be a monotonic rising function on the timeline.
        
        See documentation of binary_search.roundings for details about rounding
        options.
        '''
        
        assert issubclass(rounding, binary_search.Rounding)
        
        tree_result = self.__get_both_states_by_monotonic_function_from_tree \
                      (function, value)
        
        if tree_result[1] is not None:
            # Then there is no need to check the queue even.
            return binary_search.make_both_data_into_preferred_rounding\
                   (tree_result, function, value, rounding)
        
        else:
            queue_result = self.__get_state_by_monotonic_function_from_queue \
                           (function, value, rounding=binary_search.BOTH)
            none_count = list(queue_result).count(None)
            if none_count == 0:
                # The result is entirely in the queue
                return binary_search.make_both_data_into_preferred_rounding\
                       (queue_result, function, value, rounding)
            elif none_count == 1:
                # The result is either before the past edge of the queue or after its future edge.
                if queue_result[1] is None:
                    # The result is beyond the future edge of the queue.
                    return binary_search.make_both_data_into_preferred_rounding\
                           (queue_result, function, value, rounding)
                else: # queue_result[0] == None
                    # Getting tricky: The result is somewhere in the middle
                    # between the queue and the tree.
                    combined_result = [tree_result[0], queue_result[1]]
                    return binary_search.make_both_data_into_preferred_rounding\
                           (combined_result, function, value, rounding)
    
            else:
                assert none_count == 2
                # The queue is just totally empty.
                return binary_search.make_both_data_into_preferred_rounding \
                       (tree_result, function, value, rounding)
            
    @with_self   
    def __get_both_states_by_monotonic_function_from_tree(self, function, value):
        '''
        Get two states from the tree with value surrounding the desired value.

        The function must be a monotonic rising function on the timeline.
       
        This uses the binary_search.BOTH rounding. See its documentation.
        '''
        our_node = self.__get_our_node()
        path = our_node.make_containing_path()
        new_function = lambda node: function(node.state)
        
        result_in_nodes = path.get_node_by_monotonic_function \
            (new_function, value, binary_search.BOTH, end_node=our_node)
        
        result = [(node.state if node is not None else None) \
                  for node in result_in_nodes]
        
        return result
    
    @with_self
    def __get_state_by_monotonic_function_from_queue(
        self, function, value, rounding=binary_search.CLOSEST):
        '''
        Get a state from the queue only by measure function and desired value.
        
        The function must by a monotonic rising function on the timeline.
        
        See documentation of garlicsim.general_misc.binary_search.roundings for
        details about rounding options.
        '''
        assert issubclass(rounding, binary_search.Rounding)
        queue = self.cruncher.work_queue
        queue_as_list = queue_tools.queue_as_list(queue)
        # todo: Probably inefficient, should access them one by one
        
        return binary_search.binary_search\
               (queue_as_list, function, value, rounding)
    
    @with_self
    def __len__(self):
        '''
        Get the length of the timeline in nodes.
        
        This means the sum of:
        1. The length of the work_queue of our cruncher.
        2. The length of the path in the tree which leads to our node, up to
           our node.
        '''
        queue_length = self.cruncher.work_queue.qsize()
        
        our_node = self.__get_our_node()
        our_path = our_node.make_containing_path()
        path_length = our_path.__len__(end = our_node)
        
        return queue_length + path_length
    
    @with_self
    def __get_our_node(self):
        '''Get the node that the current cruncher is assigned to work on.'''
        jobs_to_crunchers = list(self.project.crunching_manager.crunchers.items())
        
        nodes_that_are_us = \
            [job.node for (job, cruncher) in jobs_to_crunchers \
             if cruncher == self.cruncher]
        
        num = len(nodes_that_are_us)
        assert num <= 1
        if num == 1:
            our_node = nodes_that_are_us[0]
        else: # num == 0
            raise ObsoleteCruncherError
        return our_node
        
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines tools related to copying and deepcopying operations.
'''

import copy

def deepcopy_as_simple_object(thing, memo=None):
    '''
    Deepcopy an object as a simple `object`, ignoring any __deepcopy__ method.
    '''
    if memo is None:
        memo = {}
    klass = thing.__class__
    new_thing = klass.__new__(klass)
    memo[id(thing)] = new_thing
    for (name, subthing) in vars(thing).items():
        new_thing.__dict__[name] = copy.deepcopy(subthing, memo)
    return(new_thing)
        
    




import garlicsim.data_structures


class State(garlicsim.data_structures.State):
    # This is your State subclass. Your state objects should contain all the
    # information there is about a moment of time in your simulation.
    
    def __init__(self):
        pass
    
    
    def step(self):
        # This function is the heart of your simpack. What it does is take an
        # existing world state, and output the next world state.
        #
        # This is where all the crunching gets done. This function defines the
        # laws of your simulation world.
        # 
        # The step function is one of the very few things that your simpack
        # **must** define. Almost all of the other definitions are optional.
        pass
    
        
    @staticmethod
    def create_root():
        # In this function you create a root state. This usually becomes the
        # first state in your simulation. You can make this function do
        # something simple: For example, if you're simulating Life, you can make
        # this function create an empty board.
        #
        # This function may take arguments, if you wish, to be used in making
        # the state. For example, in a Life simulation you may want to specify
        # the width and height of the board using arguments to this function.
        #
        # This function returns the newly-created state.
        pass

    
    @staticmethod
    def create_messy_root():
        # In this function you create a messy root state. This usually becomes the
        # first state in your simulation. 
        #
        # Why messy? Because sometimes you want to have fun in your simulations.
        # You want to create a world where there's a lot of mess, with many
        # objects interacting with each other. This is a good way to test-drive
        # your simulation.
        #
        # This function may take arguments, if you wish, to be used in making
        # the state. For example, in a Life simulation you may want to specify
        # the width and height of the board using arguments to this function.
        #
        # This function returns the newly-created state.
        pass
                                 
    
    # def step_generator(self):
    #     yield None
    #     pass
    #
    # Do you want to use a step generator as your step function? If so, you may
    # uncomment the above and fill it in, and it will be used instead of the
    # normal step function.
    # 
    # A step generator is similar to a regular step function: it takes a
    # starting state, and computes the next state. But it doesn't `return` it,
    # it `yield`s it. And then it doesn't exit, it just keeps on crunching and
    # yielding more states one by one.
    # 
    # A step generator is useful when you want to set up some environment and/or
    # variables when you do your crunching. It can help you save resources,
    # because you won't have to do all that initialization every time garlicsim
    # computes a step.
    #
    # (You may write your step generator to terminate at some point or to never
    # terminate-- Both ways are handled by garlicsim.)
    
    



#!/usr/bin/env python
# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Script for starting a new simpack.

Usage:
start_simpack.py quantum_mechanics
'''

import os.path
import sys
import re
import shutil

import pkg_resources

from garlicsim.scripts import simpack_template
simpack_template_package_name = simpack_template.__name__

    
def _walk_folder(package_name, folder):
    '''
    Walk on subfolders of a folder using pkg_resources.

    `package_name` is the name of the package in which this folder lives.
    `folder` is the path of the folder.
    
    Of course, since we are operating using pkg_resources, all paths are
    relative to the pkg_resources-managed package.
    '''
    folders = [folder]
    
    while folders:
        folder = folders.pop()
        for f in pkg_resources.resource_listdir(package_name, folder):
            path = '/'.join((folder, f))
            if pkg_resources.resource_isdir(package_name, path):
                folders.append(path)
            else:
                yield path
        

def _make_path_to_file(file):
    '''
    Create the folders needed before creating a file.
    
    Given a path to a file that doesn't exist, this function creates all the
    folders up to the file, so the file could be later created without thinking
    whether these folders exist or not.
    '''
    dir = os.path.split(file)[0]
    if os.path.isdir(dir):
        return
    parent_dir = os.path.split(dir)[0]
    _make_path_to_file(dir)
    os.mkdir(dir)
    
                
    
def start_simpack(containing_folder, name):
    """
    Create a new simpack.
    
    This is the main function of this module. `containing_folder` is the folder
    in which the simpack folder should be created. `name` is the name of the new
    simpack, which will also be the name of its folder.
    """
    
    if not re.search(r'^[_a-zA-Z]\w*$', name): # If not valid folder name.
        # Provide a smart error message, depending on the error.
        if not re.search(r'^[_a-zA-Z]', name):
            message = 'make sure the name begins with a letter or underscore'
        else:
            message = 'use only numbers, letters and underscores'
        raise Exception("%r is not a valid simpack name. Please %s." %
                        (name, message))
    folder = os.path.join(containing_folder, name)
    
    os.mkdir(folder)
    
    for file in _walk_folder(simpack_template_package_name, './simpack_name'):
        
        if os.path.splitext(file)[1] in ('.pyc', '.pyo'):
            continue
        
        dest_file = '/'.join(
            (containing_folder, file.replace('simpack_name', name))
        )
        
        _make_path_to_file(dest_file)
        
        with pkg_resources.resource_stream(simpack_template_package_name, file) as source:
            with open(dest_file, 'wb') as destination:
                
                string_to_write = source.read().decode('utf-8')\
                                .replace('simpack_name', name)
                
                destination.write(bytes(string_to_write, 'utf-8'))
            
        try:
            shutil.copymode('/'.join(('simpack_template', file)), dest_file)
            _make_writeable(dest_file)
        except Exception:
            pass
    print('''%s simpack created successfully! Explore the %s folder and start \
filling in the contents of your new simpack.''' % (name, name))
                
    
def _make_writeable(filename):
    """
    Make sure that the file is writeable. Useful if our source is
    read-only.
    """
    import stat
    if sys.platform.startswith('java'):
        # On Jython there is no os.access()
        return
    if not os.access(filename, os.W_OK):
        st = os.stat(filename)
        new_permissions = stat.S_IMODE(st.st_mode) | stat.S_IWUSR
        os.chmod(filename, new_permissions)

        
def show_help():
    '''Print some help text that describes how to use this script'''
    print('''\
This is a script for creating a skeleton for a garlicsim simpack. Use this when
you want to make a new simpack to have the basic folders and files created for
you.

    Usage: start_simpack.py my_simpack_name

The simpack will be created in the current path, in a directory with the name of
the simpack.''')
    return

    
if __name__ == '__main__':

    if len(sys.argv) != 2:
        show_help()
        sys.exit()
    arg = sys.argv[1]

    if arg == '--help':
        show_help()
        sys.exit()
    
    start_simpack(os.curdir, arg)
    
        



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines a function called set_process_priority. See its
documentation for more info.
'''

import  win32process, win32con, win32api

def set_process_priority(priority, pid=None):
    '''
    Set the priority of a Windows process.
    
    Priority is a value between 0-5 where 2 is normal priority. Default sets
    the priority of the current Python process but can take any valid process ID.
    '''
    
    priorityclasses = [
        win32process.IDLE_PRIORITY_CLASS,
        win32process.BELOW_NORMAL_PRIORITY_CLASS,
        win32process.NORMAL_PRIORITY_CLASS,
        win32process.ABOVE_NORMAL_PRIORITY_CLASS,
        win32process.HIGH_PRIORITY_CLASS,
        win32process.REALTIME_PRIORITY_CLASS
    ]
    
    if pid is None:
        pid = win32api.GetCurrentProcessId()
    handle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, True, pid)
    win32process.SetPriorityClass(handle, priorityclasses[priority])


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Provides decorators to cache state- or history-dependent functions.

`state_cache` is for functions that take a state. `history_cache` is for
functions that take a history browser.
'''
#todo: make sure the cache gets lost on pickling
#todo: append to function's docstring?



import weakref
import functools

import garlicsim

        

def state_cache(function):
    '''
    Caching decorator for state functions.
    
    This decorator should be used only on functions that have exactly one
    argument which is a state. (For example, in a Physics simulation you might
    want a function that takes a state and outputs the kinetic energy of that
    state.)
    
    On any subsequent calls to the function given the same state, the
    pre-calcluated value will be given from the cache instead of calculating it
    again.
    '''
    if hasattr(function, 'state_cache'): return function
    
    def cached(state):
        assert isinstance(state, garlicsim.data_structures.State)
        if state in cached.state_cache:
            return cached.state_cache[state]
        else:
            cached.state_cache[state] = value = function(state)
            return value
            
    cached.state_cache = weakref.WeakKeyDictionary()
    
    functools.update_wrapper(cached, function)
    
    return cached


def history_cache(function, *args, **kwargs):
    '''
    Caching decorator for functions that take a history browser.
    
    This decorator should be used only on functions that have exactly one
    argument which is a history browser. (For example, in a cellular automata
    simulation you might want a function that takes a history browser and tells
    you how many cells changed value between the most recent state and the one
    before it.)
    
    Note that the raw function will take a history browser, but the decorated
    function will take a node, for which a history browser will be created and
    fed into the original function.
    
    On any subsequent calls to the function given the same node, the
    pre-calcluated value will be given from the cache instead of calculating it
    again.
    '''
    if hasattr(function, 'node_cache'):
        return function
    
    def cached(node):
        assert isinstance(node, garlicsim.data_structures.Node)
        if node in cached.node_cache:
            return cached.node_cache[node]
        else:
            path = node.make_containing_path()
            with node.tree.lock.read:
                history_browser = \
                    garlicsim.synchronous_crunching.HistoryBrowser(
                        path=path,
                        end_node=node
                    )
            value = function(history_browser)
            cached.node_cache[node] = value
            return value
            
    cached.node_cache = weakref.WeakKeyDictionary()
    
    functools.update_wrapper(cached, function)
    
    return cached

if __name__ == '__main__': # make this into test
    import garlicsim
    from garlicsim_lib.simpacks import life

    s = life.make_random_state(10, 10)
    
    p = garlicsim.Project(life)
    
    r = p.root_this_state(s)
    
    p.simulate(r, 20)
    
    path = r.make_containing_path()
    
    @state_cache
    def live_cells(state):
        '''
        Meooww
        '''
        print(('calculating for state %s' % id(state)))
        return state.board._Board__list.count(True)
    
       
    for i in range(3):
        states = [node.state for node in path]
        print([live_cells(state) for state in states[-3:]])
    
    @history_cache
    def changes(history_browser):
        '''
        frrrr
        '''
        print(('calculating for hb %s' % id(history_browser)))
        try:
            state = history_browser[-1]
            last_state = history_browser[-2]
        except IndexError:
            return None
        
        board, last_board = state.board, last_state.board
        board_size = len(board._Board__list)
        counter = 0
        for i in range(board_size):
            if board._Board__list[i] != last_board._Board__list[i]:
                counter += 1
        return counter
    
    for i in range(3):
        print([changes(node) for node in path])
            
            
            



import time
import garlicsim
from garlicsim.bundled.simulation_packages import life
from garlicsim.bundled.simulation_packages import prisoner
from garlicsim.bundled.simulation_packages import _history_test


simpack = life

if __name__ == '__main__':
    
    state = simpack.make_random_state(10, 10)
    
    
    print(state)
    
    
    new_state = garlicsim.simulate(simpack, state, 10)
    
    print(new_state)
    
    result = garlicsim.list_simulate(simpack, state, 10)
    
    # assert result[-1] == new_state
    
    
    project = garlicsim.Project(simpack)
    
    #project.crunching_manager.Cruncher = \
    #    garlicsim.asynchronous_crunching.crunchers.CruncherProcess
    
    root = project.root_this_state(state)
    

    project.begin_crunching(root, 500)
    
    print((project.sync_crunchers()))
    
    time.sleep(2.0)
    print((project.sync_crunchers()))
    path = project.tree.all_possible_paths()[0]
    node = path[len(path)//2]
    new_node = project.simulate(node, 20)
    
    count = 0
    while True:
        time.sleep(0.05)
        j = project.sync_crunchers()
        print(j)
        if not project.crunching_manager.jobs:
            break
    
    
    print((root.get_all_leaves().popitem()[0].state))
    print((garlicsim.data_structures.State.__repr__(root.get_all_leaves().popitem()[0].state)))


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Infinity class and related exceptions.

See their documentation for more info.
'''

from garlicsim.misc import GarlicSimException
from garlicsim.general_misc import math_tools


__all__ = ['Infinity', 'InfinityError', 'InfinityRaceError']


def is_floatable(x):
    try:
        float(x)
        return True
    except:
        return False

def is_nonfractional(x):
    try:
        int(x)
        return int(x) == x
    except:
        return False
    
class InfinityRaceError(GarlicSimException):
    '''
    An "infinity race" between two infinite sizes.
    
    A calculation is being made between two quantities that involve infinity,
    and the two infinities are "pitted" against each other in a way which makes
    it impossible to determine what the result of the computation would be.
    '''

class InfinityError(GarlicSimException):
    '''Infinity-related exception.'''

    
class InfinityClass(object):
    '''
    A class for infinity numbers.
    
    There are only two distinct instances of this class: Infinity and
    (-Infinity).
    '''
    #todo: add __assign__ or whatever it's called
    #todo: add more interoperability with float(inf). (Need to detect its
    #existance)    
    #todo: calling it InfinityClass is a bit wrong./
    
    def __init__(self, direction=1):
        self.direction = direction
        
    def __abs__(self):
        return Infinity
    
    def __add__(self, other):
        if isinstance(other, InfinityClass):
            if self.direction == other.direction:
                return self
            else:
                raise InfinityRaceError
        elif is_floatable(other):
            return self
        
    def __sub__(self, other):
        return self.__add__(-other)
    
    def __cmp__(self, other):
        if isinstance(other, InfinityClass):
            d_cmp = math_tools.cmp(self.direction, other.direction)
            if d_cmp != 0:
                return d_cmp
            else:
                raise InfinityRaceError
        elif is_floatable(other):
            return self.direction
        else:
            raise NotImplementedError
        
    def __lt__(self, other):
        return self.__cmp__(other) == -1
    
    def __le__(self, other):
        return self.__cmp__(other) != 1

    def __gt__(self, other):
        return self.__cmp__(other) == 1

    def __ge__(self, other):
        return self.__cmp__(other) != -1
            
    def __div__(self, other):
        if isinstance(other, InfinityClass):
            raise InfinityRaceError
        elif is_floatable(other):
            s = math_tools.sign(other)
            if s==0:
                raise InfinityRaceError
            else:
                return InfinityClass(direction=self.direction * s)
            
    def __float__(self):
        raise ValueError("Can't convert infinite number to float")
    
    def __mul__(self, other):
        if isinstance(other, InfinityClass):
            return InfinityClass(self.direction * other.direction)
        elif is_floatable(other):
            s = math_tools.sign(other)
            if s==0:
                raise InfinityRaceError
            else:
                return InfinityClass(direction=self.direction * s)
            
    def __neg__(self):
        return InfinityClass(-self.direction)
    
    def __bool__(self):
        return True
    
    def __pos__(self):
        return self
    
    def __pow__(self, other):
        if isinstance(other, InfinityClass):
            raise object # todo
        elif is_floatable(other):
            s = math_tools.sign(other)
            if s==0:
                raise InfinityRaceError
            else:
                if self.direction==1:
                    if s==1:
                        return self
                    if s==-1:
                        return 0
                else: #self.direction == -1
                    if is_nonfractional(other):
                        if s==-1:
                            return 0
                        if s==1:
                            if s % 2 == 0:                                
                                return InfinityClass()
                            else:                    
                                return InfinityClass(-1)                            
                    else: # is_nonfractional(other) is False
                        raise ValueError(""""negative number cannot be raised
to a fractional power""")            
          
                
    def __rpow__(self, other):
        if isinstance(other, InfinityClass):
            return other.__pow__(self)
        elif not is_floatable(other):
            raise NotImplementedError
        else: # is_floatable(number) is True
            raise NotImplementedError # todo
        
    def __radd__(self, other):
        return self.__add__(other)
    
    def __rsub__(self, other):
        return ( - self.__sub__(other))
    
    def __rmul__(self, other):
        return self.__mul__(other)
    
    __truediv__ = __floordiv__ = __div__
    
    def __eq__(self, other):

        if isinstance(other, InfinityClass):
            return other.direction == self.direction
        
        elif isinstance(other, float):
            # We're checking to see if `other` is equal to `float('inf')` or
            # `-float('inf')`. But we must `try` it carefully, because in Python
            # 2.5 there is no `float('inf')`.
            #
            # Todo: It seems this takes precedence over `float.__eq__`,
            # fortunately. How come this happens?
            try:
                float_inf = float('inf')
            except ValueError:
                return False
            
            if other == float_inf:
                return self.direction == 1
            elif other == -float_inf:
                return self.direction == -1
            else:
                return False
            
        else: # other is not any kind of infinity
            return False
        
    
    def __neq__(self, other):
        return not self.__eq__(other)
    
    def __repr__(self):
        if self.direction==1:
            suffix=''
        else: # self.direction == -1
            suffix='-'
        return suffix + 'Infinity'
        

Infinity = InfinityClass()





import wx

import garlicsim_wx


class StateViewer(wx.Panel,
                  garlicsim_wx.widgets.WorkspaceWidget):
    # Here you create a widget that will display your state graphically on the
    # screen.
    
    def __init__(self, frame):
            
        # We need to call the __init__ of both our base classes:
        wx.Panel.__init__(self, frame,
                                        style=wx.SUNKEN_BORDER)        
        garlicsim_wx.widgets.WorkspaceWidget.__init__(self, frame)
        
        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM) # Solves Windows flicker
        
        self.Bind(wx.EVT_PAINT, self.on_paint)
        self.Bind(wx.EVT_SIZE, self.on_size)
        
        # ...
        

        # This next bit will cause the widget to get updated every time the
        # active state in the GUI is changed:
        self.gui_project.active_node_changed_emitter.add_output(
            lambda: self.set_state(self.gui_project.get_active_state())
        )


    def set_state(self, state):
        # Here you set the state to be displayed.
        self.state = state
        self.Refresh()
        
        
    def on_paint(self, event):
        # This is your EVT_PAINT handler, which draws the state on the widget.
        
        event.Skip()
        
        dc = wx.PaintDC(self)
        
        # ...
        
        dc.Destroy()
        
        
    def on_size(self, event):
        # An EVT_SIZE handler. Just some wxPython thing that I think you're
        # supposed to do.
        self.Refresh()




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Package for caching of functions that take states or history browsers.'''

from .decorators import state_cache, history_cache


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CoolDict class. See its documentation for more
information.
'''

# remove has_key from all places

import copy

class CoolDict(dict):
    '''
    A class derived from dict which defines some extra methods.
    '''
    def raise_to(self, key, value):
        '''
        Same as `cool_dict[key] = value`, except if the cool dict already has
        `key`, and its value is higher than `value`.
        '''
        has_key = key in self
        if not has_key:
            self[key] = value
        else:
            self[key] = max(value, self[key])
            
    def lower_to(self, key, value):
        '''
        Same as `cool_dict[key] = value`, except if the cool dict already has
        `key`, and its value is lower than `value`.
        '''
        has_key = key in self
        if not has_key:
            self[key] = value
        else:
            self[key] = min(value, self[key])
            
    def copy(self):
        return CoolDict(self)
    
    def __copy__(self):
        return CoolDict(self)
    
    def __deepcopy__(self, memo):
        raise NotImplementedError # todo
    
    def transfer_value(self, key, new_key):
        assert key in self
        assert new_key not in self
        
        value = self[key]
        self[new_key] = value
        del self[key]
        
        return value


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the list_simulate function.

See its documentation for more info.
'''

import copy

from garlicsim.general_misc import cute_iter_tools

import garlicsim
import garlicsim.misc
from . import history_browser as history_browser_module # Avoiding name clash

__all__ = ["list_simulate"]


def list_simulate(state, iterations, *args, **kwargs):
    '''
    Simulate from the given state for the given number of iterations.

    Any extraneous parameters will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    simpack_grokker = garlicsim.misc.SimpackGrokker.create_from_state(state)
    step_profile = garlicsim.misc.StepProfile(*args, **kwargs)
    
    if not hasattr(state, 'clock'):
        state = copy.deepcopy(state,
                              garlicsim.misc.persistent.DontCopyPersistent())
        state.clock = 0
                      
    if simpack_grokker.history_dependent:
        return _history_list_simulate(simpack_grokker, state, iterations,
                                       step_profile)
    else: # It's a non-history-dependent simpack
        return _non_history_list_simulate(simpack_grokker, state, iterations,
                                           step_profile)

    
def _history_list_simulate(simpack_grokker, state, iterations,
                             step_profile=None):
    '''
    Simulate from the given state for the given number of iterations.

    (Internal function for history-dependent simulations only.)    
    
    A simpack grokker must be passed as the first parameter. Any extraneous
    parameters will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    
    if step_profile is None: step_profile = garlicsim.misc.StepProfile()
    
    tree = garlicsim.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    history_browser = history_browser_module.HistoryBrowser(path)
    
    iterator = simpack_grokker.step_generator(history_browser, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    current_state = current_node.state
    
    world_ended = False
    try:
        for current_state in finite_iterator:
            current_node = tree.add_state(current_state, parent=current_node)
    except garlicsim.misc.WorldEnd:
        world_ended = True
    
    # Not doing anything with `world_ended` yet
        
    return [node.state for node in path]


def _non_history_list_simulate(simpack_grokker, state, iterations,
                                step_profile=None):
    '''
    Simulate from the given state for the given number of iterations.
    
    (Internal function for non-history-dependent simulations only.)

    A simpack grokker must be passed as the first parameter. Any extraneous
    parameters will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''

    if step_profile is None: step_profile = garlicsim.misc.StepProfile()
    
    tree = garlicsim.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    
    iterator = simpack_grokker.step_generator(state, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    current_state = current_node.state
    
    world_ended = False
    try:
        for current_state in finite_iterator:
            current_node = tree.add_state(current_state, parent=current_node)
    except garlicsim.misc.WorldEnd:
        world_ended = True

    # Not doing anything with `world_ended` yet
    
    return [node.state for node in path]




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the BaseHistoryBrowser class.

See its documentation for more info.
'''

import garlicsim.general_misc.third_party.abc as abc
from garlicsim.general_misc import binary_search

__all__ = ["HistoryBrowser"]

get_state_clock = lambda state: state.clock

class BaseHistoryBrowser(object, metaclass=abc.ABCMeta):
    '''
    A device for requesting information about the history of the simulation.

    With a HistoryBrowser one can request states from the simulation's timeline.
    States can be requested by clock time or position in the timeline or by
    other measures; See documentation for this class's methods.
    
    This is an abstract base class from which all history browsers should
    subclass.
    '''
    
    @abc.abstractmethod
    def get_last_state(self):
        '''Get the last state in the timeline. Identical to __getitem__(-1).'''
        pass
    
    @abc.abstractmethod
    def __getitem__(self, i):
        '''Get a state by its position in the timeline.'''
        pass
    
    @abc.abstractmethod
    def get_state_by_monotonic_function(self, function, value, rounding):
        '''
        Get a state by specifying a measure function and a desired value.
        
        The function must be a monotonic rising function on the timeline.
        
        See documentation of garlicsim.general_misc.binary_search.roundings for
        details about rounding options.
        '''
        pass
    
    @abc.abstractmethod
    def __len__(self):
        '''Get the length of the timeline in nodes.'''
        pass
    
    
    def get_state_by_clock(self, clock, rounding=binary_search.CLOSEST):
        '''
        Get a state by specifying desired clock time.
        
        See documentation of garlicsim.general_misc.binary_search.roundings for
        details about rounding options.
        '''
        assert issubclass(rounding, binary_search.Rounding)
        return self.get_state_by_monotonic_function\
               (function=get_state_clock, value=clock, rounding=rounding)


import time
import garlicsim
from garlicsim.bundled.simulation_packages import life
from garlicsim.bundled.simulation_packages import prisoner

simpack = life

if __name__ == '__main__':
    
    state = simpack.make_random_state(10, 10)
    
    
    print(state)
    
    
    new_state = garlicsim.simulate(simpack, state, 10)
    
    print(new_state)
    
    result = garlicsim.list_simulate(simpack, state, 10)
    
    assert result[-1] == new_state
    
    print([thing.clock for thing in result])


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CrunchingManager class. See its documentation for more
information.
'''


import garlicsim.general_misc.queue_tools as queue_tools
import garlicsim.general_misc.third_party.decorator
import garlicsim.general_misc.change_tracker
from garlicsim.general_misc.infinity import Infinity
from garlicsim.general_misc import misc_tools
from garlicsim.general_misc import cute_iter_tools

import garlicsim
import garlicsim.data_structures
import garlicsim.misc
from . import crunchers
from .crunching_profile import CrunchingProfile
from garlicsim.misc.step_profile import StepProfile
from .misc import EndMarker

__all__ = ['CrunchingManager', 'DefaultCruncher', 'DefaultHistoryCruncher']


DefaultCruncher = crunchers.CruncherThread
'''Cruncher class to be used by default in non-history-dependent simulations.'''


DefaultHistoryCruncher = crunchers.CruncherThread
'''Cruncher class to be used by default in history-dependent simulations.'''


@garlicsim.general_misc.third_party.decorator.decorator
def with_tree_lock(method, *args, **kwargs):
    '''
    Decorator for using the tree lock (in write mode) as a context manager.
    '''
    self = args[0]
    with self.project.tree.lock.write:
        return method(*args, **kwargs)


class CrunchingManager(object):
    '''
    A crunching manager manages the background crunching for a project.
    
    Every project creates a crunching manager. The job of the crunching manager
    is to coordinate the crunchers, creating and retiring them as necessary.
    The main use of a crunching manager is through its sync_workers methods,
    which goes over all the crunchers and all the nodes of the tree that need
    to be crunched, making sure the crunchers are working on these nodes, and
    collecting work from them to implement into the tree.
    
    The crunching manager contains a list of jobs as an attribute `.jobs`. See
    documentation for garlicsim.asynchronous_crunching.Job for more info about
    jobs. The crunching manager will employ crunchers in order to complete the
    jobs. It will then take work from these crunchers, put it into the tree,
    and delete the jobs when they are completed.
    '''
    def __init__(self, project):        
        self.project = project
        

        FORCE_CRUNCHER = project.simpack_grokker.settings.FORCE_CRUNCHER
        
        if FORCE_CRUNCHER is not None:
            self.Cruncher = FORCE_CRUNCHER
        else:
            history_dependent = project.simpack_grokker.history_dependent
            
            self.Cruncher = DefaultHistoryCruncher if history_dependent \
                            else DefaultCruncher
        
        self.jobs = []
        '''
        The jobs that the crunching manager will be responsible for doing.
        
        These are of the class garlicsim.asynchronous_crunching.Job.
        '''
        
        self.crunchers = {}
        '''Dict that maps each job to the cruncher reponsible for doing it.'''
        
        self.step_profiles = {}
        '''
        Dict that maps each cruncher to its step options profile.
        
        This exists because a cruncher might change its step options profile
        in the course of its work. When it does, it announces this by putting
        the profile in the work queue. In this dict we keep track of the last
        step options profile each cruncher was known to use.
        '''
        
        self.crunching_profiles_change_tracker = \
            garlicsim.general_misc.change_tracker.ChangeTracker()
        '''
        A change tracker which tracks changes made to crunching profiles.
        
        This is used to update the crunchers if the crunching profile for the
        job they're working on has changed.
        '''

        
    @with_tree_lock
    def sync_crunchers(self):
        '''
        Take work from the crunchers, and give them new instructions if needed.
        
        Talks with all the crunchers, takes work from them for implementing
        into the tree, retiring crunchers or recruiting new crunchers as
        necessary.

        Returns the total amount of nodes that were added to the tree in the
        process.
        '''
        # Danger, Will Robinson! This right here is one of the most technical
        # and sensitive functions in all of GarlicSim-land. It's hard to keep
        # track of what's going on in this function. Be careful if you're trying
        # to make changes. May the Force be with you.
        
        # todo: Extensively document the implementation of this function.
        
        total_added_nodes = garlicsim.misc.NodesAdded(0)

        
        for (job, cruncher) in list(self.crunchers.copy().items()):
            if not (job in self.jobs):
                (added_nodes, new_leaf) = \
                    self.__add_work_to_tree(cruncher, job, retire=True)
                total_added_nodes += added_nodes
                del self.crunchers[job]


        for job in self.jobs[:]:
            
            
            if job not in self.crunchers:
                if not job.is_done():
                    self.__conditional_create_cruncher(job)
                else: # job.is_done() is True
                    self.jobs.remove(job)
                continue

            # job in self.crunchers
            
            cruncher = self.crunchers[job]
            
            (added_nodes, new_leaf) = self.__add_work_to_tree(cruncher,
                                                              job)
            total_added_nodes += added_nodes

            job.node = new_leaf
            
            if not job.is_done(): # (Need to call is_done again cause node changed)
                
                crunching_profile = job.crunching_profile
                
                if cruncher.is_alive():
                                        
                    if self.crunching_profiles_change_tracker.check_in \
                       (crunching_profile):
                        
                        cruncher.update_crunching_profile(crunching_profile)
                        
                else:
                    
                    self.__conditional_create_cruncher(job)
            else: # job.is_done() is True
                self.jobs.remove(job)
                if cruncher.is_alive():
                    cruncher.retire()
                del self.crunchers[job]
            
        return total_added_nodes

    
    def __conditional_create_cruncher(self, job):
        '''
        Create a cruncher to crunch the node, unless there is reason not to.
        '''
        node = job.node
        crunching_profile = job.crunching_profile
        
        if node.still_in_editing is False:
            step_function = self.project.simpack_grokker.step
            if self.Cruncher == getattr(crunchers, 'CruncherProcess', None):
                cruncher = self.Cruncher \
                         (node.state,
                          self.project.simpack_grokker.step_generator,
                          crunching_profile=crunching_profile)
            else: # self.Cruncher == crunchers.CruncherThread
                cruncher = self.Cruncher(node.state, self.project,
                                         crunching_profile=crunching_profile)
            cruncher.start()
            self.crunchers[job] = cruncher
            
            self.crunching_profiles_change_tracker.check_in(crunching_profile)
            self.step_profiles[cruncher] = \
                crunching_profile.step_profile
            
    
    def get_jobs_by_node(self, node):
        '''
        Get all the jobs that should be done on the specified Node.
        
        This is every job whose .node attribute is equal to the specified node.
        '''
        return [job for job in self.jobs if job.node == node]

    
    def __add_work_to_tree(self, cruncher, job, retire=False):
        '''
        Take work from cruncher and add to tree at the specified job's node.
        
        If `retire` is set to True, retires the cruncher. Keep in mind that if
        the cruncher gives an EndMarker, it will be retired regardless of the
        `retire` argument.
        
        Returns (number, leaf), where `number` is the number of nodes that were
        added, and `leaf` is the last node that was added.
        '''
        
        tree = self.project.tree
        node = job.node
        
        current = node
        counter = 0
        self.step_profiles[cruncher]
        
        for thing in queue_tools.iterate(cruncher.work_queue):
            
            if isinstance(thing, garlicsim.data_structures.State):
                counter += 1
                current = tree.add_state(
                    thing,
                    parent=current,
                    step_profile=self.step_profiles[cruncher],
                    
                )
                # todo optimization: save step profile in variable, it's
                # wasteful to do a dict lookup every state.
            
            elif isinstance(thing, EndMarker):
                tree.make_end(node=current,
                              step_profile=self.step_profiles[cruncher])
                job.resulted_in_end = True
                
                
            elif isinstance(thing, StepProfile):
                self.step_profiles[cruncher] = thing
            else:
                raise Exception('Unexpected object %s in work queue' % thing)
                        
        if retire or job.resulted_in_end:
            cruncher.retire()
        
        nodes_added = garlicsim.misc.NodesAdded(counter)

        return (nodes_added, current)
    
    def __repr__(self):
        '''
        Get a string representation of the crunching manager.
        
        Example output:
        <garlicsim.asynchronous_crunching.CrunchingManager
        currently employing 2 crunchers to handle 2 jobs at 0x1f699b0>
        '''
        
        crunchers_count = len(self.crunchers)
        job_count = len(self.jobs)
                                   
        return '<%s currently employing %s crunchers to handle %s jobs at %s>' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                   ),
                   crunchers_count,
                   job_count,
                   hex(id(self))
               )
    
    
    
    



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This module defines logic-related tools.'''

import garlicsim.general_misc.cute_iter_tools as cute_iter_tools


def all_equal(iterable, exhaustive=False):
    '''
    Return whether all elements in the iterable are equal to each other.
    
    If `exhaustive` is set to False, it's assumed that the equality relation is
    transitive, therefore not every member is tested against every other member.
    So in a list of size n, n-1 equality checks will be made.
    
    If `exhaustive` is set to True, every member will be checked against every
    other member. So in a list of size n, (n*(n-1))/2 equality checks will be
    made.
    '''
    
    if exhaustive is True:
        pairs = cute_iter_tools.orderless_combinations(iterable, 2)        
    else: # exhaustive is False
        pairs = cute_iter_tools.consecutive_pairs(iterable)
        
    return all(a==b for (a, b) in pairs)


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Persistent class and some related classes.

See documentation of Persistent and CrossProcessPersistent for more info.
'''

from .persistent import Persistent
from .cross_process_persistent import CrossProcessPersistent
from .personality import Personality
from .copy_modes import DontCopyPersistent



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CrunchingProfile class.

See its documentation for more information.
'''

import garlicsim.misc

__all__ = ['CrunchingProfile']

class CrunchingProfile(object):
    '''Instructions that a cruncher follows when crunching the simulation.'''
    
    def __init__(self, clock_target=None, step_profile=None):
        '''
        Construct the CrunchingProfile.
        
        `clock_target` is the clock until which we want to crunch.
        `step_profile` is the step profile we want to use.
        '''
        
        self.clock_target = clock_target
        '''
        We crunch until we get a state with a clock of `.clock_target` or higher.
        '''
        
        self.step_profile = step_profile or garlicsim.misc.StepProfile()
        '''The step profile we want to be used with the step function.'''
  
    def state_satisfies(self, state):
        '''
        Check whether a state has a clock high enough to satisfy this profile.
        '''
        return state.clock >= self.clock_target
    
    def raise_clock_target(self, clock_target):
        '''If .clock_target is lower than the given clock_target, raise it.'''
        if self.clock_target < clock_target:
            self.clock_target = clock_target
    
    def __eq__(self, other):
        return isinstance(other, CrunchingProfile) and \
               self.clock_target == other.clock_target and \
               self.step_profile == other.step_profile

    def __hash__(self):
        # Defining __hash__ because there's __eq__ which makes the default
        # __hash__ disappear on Python 3.
        return id(self)

    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __repr__(self):
        '''
        Get a string representation of the crunching profile.
        
        Example output:
        CrunchingProfile(clock_target=Infinity, step_profile=StepProfile())
        '''
        stuff = []
        stuff.append("clock_target=%s" % self.clock_target)
        stuff.append("step_profile=%s" % self.step_profile)
        temp = ", ".join(stuff)
        return ("CrunchingProfile(%s)" % temp)
    
    
    
    
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This module defines miscellaneous tools.'''

import math


def frange(start, finish=None, step=1.):
    '''
    Make a list containing an arithmetic progression of numbers.

    This is an extension of the builtin `range`; It allows using floating point
    numbers.
    '''
    if finish is None:
        finish, start = start, 0.
    else:
        start = float(start)

    count = int(math.ceil(finish - start)/step)
    return (start + n*step for n in range(count))


def shorten_class_address(module_name, class_name):
    '''
    Shorten the address of a class.
    
    This is mostly used in `__repr__` methods of various classes to shorten the
    text and make the final output more conscise. For example, if you have a
    class `garlicsim.asynchronous_crunching.project.Project`, but which is also
    available as `garlicsim.Project`, this function will return
    'garlicsim.Project'.    
    '''
    get_module = lambda module_name: __import__(module_name, fromlist=[''])
    original_module = get_module(module_name)
    original_class = getattr(original_module, class_name)
    
    current_module_name = module_name
    
    last_successful_module_name = current_module_name
    
    while True:
        # Removing the last submodule from the module name:
        current_module_name = '.'.join(current_module_name.split('.')[:-1]) 
        
        if not current_module_name:
            # We've reached the top module and it's successful, can break now.
            break
        
        current_module = get_module(current_module_name)
        
        candidate_class = getattr(current_module, class_name, None)
        
        if candidate_class is original_class:
            last_successful_module_name = current_module_name
        else:
            break
        
    return '.'.join((last_successful_module_name, class_name))


class LazilyEvaluatedConstantProperty(object):
    '''
    A property that is calculated (a) lazily and (b) only once for an object.
    
    Usage:
    
        class MyObject(object):
        
            # ... Regular definitions here
        
            def _get_personality(self):
                print('Calculating personality...')
                time.sleep(5) # Time consuming process that creates personality
                return 'Nice person'
        
            personality = LazilyEvaluatedConstantProperty(_get_personality)
    
    '''
    def __init__(self, getter, name=None):
        '''
        Construct the LEC-property.
        
        You may optionally pass in the name the this property has in the class;
        This will save a bit of processing later.
        '''
        self.getter = getter
        self.our_name = name
        
        
    def __get__(self, obj, our_type=None):

        value = self.getter(obj)
        
        if not self.our_name:
            if not our_type:
                our_type = type(obj)
            (self.our_name,) = (key for (key, value) in 
                                vars(our_type).items()
                                if value is self)
        
        setattr(obj, self.our_name, value)
        
        return value
        
        
if __name__ == '__main__': # todo: move to test suite
    import random
    class A(object):
        def _get_personality(self):
            print(
                "Calculating personality for %s. (Should happen only once.)" % self
            )
            return random.choice(['Angry', 'Happy', 'Sad', 'Excited'])
        personality = LazilyEvaluatedConstantProperty(_get_personality)
    a = A()
    print(a.personality)
    print(a.personality)
    print(a.personality)
    
    a2 = A()
    print(a2.personality)
    print(a2.personality)
    print(a2.personality)
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This module defines math-related tools.'''


def sign(x):
    '''Get the sign of a number.'''
    if x > 0:
        return 1
    if x == 0:
        return 0
    assert x < 0
    return -1


def cmp(a, b):
    '''
    Compare two objects.
    
    Returns 1 if a > b, returns -1 if b > a, returns 0 if they're equal.
    '''
    return (a > b) - (a < b)


def round_to_int(x, up=False):
    '''
    Round a number to an int.
    
    This is mostly used for floating points. By default, it will round the
    number down, unless the `up` argument is set to True and then it will round
    up.
    
    If you want to round a number to the closest int, just use int(round(x)).
    '''
    rounded_down = int(x // 1)
    if up:
        return int(x) if x.is_integer() else rounded_down + 1
    else:
        return rounded_down



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

from garlicsim.data_structures import Tree, Node, Block

#todo

def show(tree):
    if len(tree.roots) > 1: raise NotImplementedError
    return _show(tree.roots[0])


def _show(thing):
    pass


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Project class.

See its documentation for more information.
'''


from garlicsim.general_misc import cute_iter_tools
from garlicsim.general_misc import misc_tools
import garlicsim.general_misc.read_write_lock
from garlicsim.general_misc.infinity import Infinity
import garlicsim.general_misc.module_wrapper
import garlicsim.general_misc.third_party.decorator

import garlicsim.data_structures
import garlicsim.misc.simpack_grokker
import garlicsim.misc.step_profile

from .crunching_manager import CrunchingManager
from .job import Job
from .crunching_profile import CrunchingProfile

__all__ = ["Project"]

@garlicsim.general_misc.third_party.decorator.decorator
def with_tree_lock(method, *args, **kwargs):
    '''
    A decorator used in Project's methods to use the tree lock (in write mode)
    as a context manager when calling the method.
    '''
    self = args[0]
    with self.tree.lock.write:
        return method(*args, **kwargs)


class Project(object):
    '''
    A simulation project. This is the flagship class of `garlicsim`.
    
    You create a project when you want to do a simulation which will crunch
    in the background with worker threads or worker processes.

    A project contains within it a tree.
        
    The crunching is taken care of by the CrunchingManager which is a part of
    every project. The CrunchingManager employs CruncherThreads and/or
    CruncherProcesses to get the work done. To make the CrunchingManager take
    work from the crunchers and coordinate them, call the sync_crunchers method
    of the project.
    
    The crunching manager contains a list of jobs as an attribute `.jobs`. See
    documentation for garlicsim.asynchronous_crunching.Job for more info about
    jobs. The crunching manager will employ crunchers in order to complete the
    jobs. It will then take work from these crunchers, put it into the tree,
    and delete the jobs when they are completed.
    '''

    def __init__(self, simpack):
        
        if isinstance(simpack, garlicsim.misc.SimpackGrokker):
            self.simpack_grokker = simpack            
            self.simpack = self.simpack_grokker.simpack
        else: # The simpack we got is an actual simpack
            self.simpack = simpack
            self.simpack_grokker = garlicsim.misc.SimpackGrokker(simpack)

        self.tree = garlicsim.data_structures.Tree()
        
        self.crunching_manager = CrunchingManager(self)
    

    def create_root(self, *args, **kwargs):
        '''
        Create a parentless node whose state is a simple plain state.
        
        The simulation package has to define the method `State.create_root` for
        this to work.
        
        Returns the node.
        '''
        state = self.simpack.State.create_root(*args, **kwargs)
        return self.root_this_state(state)

    
    def create_messy_root(self, *args, **kwargs):
        '''
        Creates a parentless node whose state is a random and messy state.
        
        The simulation package has to define the method
        `State.create_messy_root` for this to work.
        
        Returns the node.
        '''
        state = self.simpack.State.create_messy_root(*args, **kwargs)
        return self.root_this_state(state)

    
    def root_this_state(self, state):
        '''
        Take a state, wrap it in a node and add to the tree without a parent.
        
        Returns the node.
        '''
        return self.tree.add_state(state)

    
    def ensure_buffer(self, node, clock_buffer=0):
        '''
        Ensure there's a large enough buffer of nodes after `node`.

        This method will ensure that every path that starts at `node` will have
        a clock buffer of at least `clock_buffer` after `node`. If there isn't,
        the leaves of `node` will be crunched until there's a buffer of
        `clock_buffer` between `node` and each of the leaves.
        '''
        leaves_dict = node.get_all_leaves(max_clock_distance=clock_buffer)
        new_clock_target = node.state.clock + clock_buffer
        
        for leaf in leaves_dict.copy():
            if leaf.ends: # todo: Not every end should count.
                del leaves_dict[leaf]
        
        for item in leaves_dict.items():

            leaf = item[0]
            
            jobs_of_leaf = self.crunching_manager.get_jobs_by_node(leaf)
            
            if not jobs_of_leaf:
                step_profile = leaf.step_profile or garlicsim.misc.StepProfile()
                crunching_profile = CrunchingProfile(new_clock_target,
                                                     step_profile)
                job = Job(leaf, crunching_profile)
                self.crunching_manager.jobs.append(job)
                continue
            
            for job in jobs_of_leaf:
                job.crunching_profile.raise_clock_target(new_clock_target)
            
    
    def ensure_buffer_on_path(self, node, path, clock_buffer=0):
        '''
        Ensure there's a large enough buffer of nodes after `node` on `path`.

        This method will ensure that on the given path there will be a clock
        buffer of at least `clock_buffer` after `node`. If there isn't, the
        leaf at the end of the path will be crunched until the buffer is big
        enough.
        '''
        
        leaf = path.get_last_node(start=node)
        if leaf.ends: # todo: Not every end should count, I think.
            return
        
        new_clock_target = node.state.clock + clock_buffer     

        jobs_of_leaf = self.crunching_manager.get_jobs_by_node(leaf)
        
        if jobs_of_leaf:
            job = jobs_of_leaf[-1]
            # We only want to take one job. We're guessing the last, and 
            # therefore the most recent one, will be the most wanted by the
            # user.
            job.crunching_profile.raise_clock_target(new_clock_target)
            return job
        else:
            step_profile = leaf.step_profile or garlicsim.misc.StepProfile()
            crunching_profile = CrunchingProfile(new_clock_target,
                                                 step_profile)
            job = Job(leaf, crunching_profile)
            self.crunching_manager.jobs.append(job)
            return job

        
    def fork_to_edit(self, template_node):
        '''
        "Duplicate" the node, marking the new one as touched.
        
        The new node will have the same parent as `template_node`. The state of
        the new node is usually modified by the user after it is created, and
        after that the node is finalized and used in simulation.
        
        This is useful when you want to make some changes in the world state and
        see what they will cause in the simulation.
        
        Returns the node.
        '''
        return self.tree.fork_to_edit(template_node)

    
    def begin_crunching(self, node, clock_buffer=None, *args, **kwargs):
        '''
        Start a new crunching job from `node`, possibly forking the simulation.
        
        On the next call to .sync_crunchers, a cruncher will start working on
        the new job.
        
        If there are already jobs on that node, they will all be crunched
        independently of each other to create different forks.
        
        Any args or kwargs will be packed in a StepProfile object and passed to
        the step function. You may pass a StepProfile yourself, as the only
        argument, and it will be noticed and used.
        
        Returns the job.
        '''
        
        step_profile = garlicsim.misc.StepProfile(*args, **kwargs)
        
        clock_target = node.state.clock + clock_buffer
        
        crunching_profile = CrunchingProfile(clock_target, step_profile)
        
        job = Job(node, crunching_profile)
        
        return self.crunching_manager.jobs.append(job)
    

    def sync_crunchers(self):
        '''
        Take work from the crunchers, and give them new instructions if needed.
        
        Talks with all the crunchers, takes work from them for implementing
        into the tree, retiring crunchers or recruiting new crunchers as
        necessary.

        Returns the total amount of nodes that were added to the tree in the
        process.
        '''        
        return self.crunching_manager.sync_crunchers()
    
    
    @with_tree_lock
    def simulate(self, node, iterations=1, *args, **kwargs):
        '''
        Simulate from the given node for the given number of iterations.
        
        The results are implemented the results into the tree. Note that the
        crunching for this is done synchronously, i.e. in the currrent thread.
        
        Any extraneous parameters will be passed to the step function.
        
        Returns the final node.
        '''
        # todo: is simulate a good name? Need to say it's synchronously
        
        step_profile = garlicsim.misc.StepProfile(*args, **kwargs)
        
        if self.simpack_grokker.history_dependent:
            return self.__history_dependent_simulate(node, iterations,
                                                     step_profile)
        else:
            return self.__non_history_dependent_simulate(node, iterations,
                                                         step_profile)
        
        
    @with_tree_lock        
    def __history_dependent_simulate(self, node, iterations,
                                     step_profile=None):
        '''
        Simulate from the given node for the given number of iterations.
        
        (Internal function for history-dependent simulations only.)
        
        The results are implemented the results into the tree. Note that the
        crunching for this is done synchronously, i.e. in the currrent thread.
        
        A step profile may be passed to be used with the step function.
        
        Returns the final node.
        '''
        
        if step_profile is None: step_profile = garlicsim.misc.StepProfile()
        
        path = node.make_containing_path()
        history_browser = \
            garlicsim.synchronous_crunching.HistoryBrowser(path, end_node=node)
        
        iterator = self.simpack_grokker.step_generator(history_browser,
                                                       step_profile)
        finite_iterator = cute_iter_tools.shorten(iterator, iterations)
        
        current_node = node
        first_run = True
        try:
            for current_state in finite_iterator:
                current_node = self.tree.add_state(current_state,
                                                   parent=current_node,
                                                   step_profile=step_profile)
                history_browser.end_node = current_node
                if first_run:
                    history_browser.path = current_node.make_containing_path()
                    # Just once, after the first run, we set the path of the
                    # history browser to be the new end_node's path. Why?
                    
                    # Because just after the first run we've created the first
                    # new node, possibly causing a fork. Because of the new
                    # fork, the original path that we created at the beginning
                    # of this method will get confused and take the old timeline
                    # instead of the new timeline. (And it wouldn't even have
                    # the end_node to stop it, because that would be on the new
                    # timeline.) So we create a new path for the history
                    # browser. We only need to do this once, because after the
                    # first node we work on one straight timeline and we don't
                    # fork the tree any more.
        
        except garlicsim.misc.WorldEnd:
            self.tree.make_end(current_node, step_profile)
            
        return current_node
    
    
    @with_tree_lock
    def __non_history_dependent_simulate(self, node, iterations,
                                         step_profile=None):
        '''
        Simulate from the given node for the given number of iterations.
        
        (Internal function for non-history-dependent simulations only.)
        
        The results are implemented the results into the tree. Note that the
        crunching for this is done synchronously, i.e. in the currrent thread.
        
        A step profile may be passed to be used with the step function.
        
        Returns the final node.
        '''
        
        if step_profile is None: step_profile = garlicsim.misc.StepProfile()

        state = node.state
                
        iterator = self.simpack_grokker.step_generator(state, step_profile)
        finite_iterator = cute_iter_tools.shorten(iterator, iterations)
        
        current_node = node
        
        try:
            for current_state in finite_iterator:
                current_node = self.tree.add_state(current_state,
                                                   parent=current_node,
                                                   step_profile=step_profile)
        except garlicsim.misc.WorldEnd:
            self.tree.make_end(current_node, step_profile)
            
        return current_node
    
    
    def iter_simulate(self, node, iterations=1, *args, **kwargs):
        '''
        Simulate from the given node for the given number of iterations.
        
        The results are implemented into the tree. Note that the crunching for
        this is done synchronously, i.e. in the currrent thread.
        
        This returns a generator that yields all the nodes one-by-one, from the
        initial node to the final one.
        
        Any extraneous parameters will be passed to the step function.
        '''
        
        step_profile = garlicsim.misc.StepProfile(*args, **kwargs)
        
        if self.simpack_grokker.history_dependent:
            return self.__history_dependent_iter_simulate(node, iterations,
                                                          step_profile)
        else:
            return self.__non_history_dependent_iter_simulate(node, iterations,
                                                              step_profile)
        
                
    def __history_dependent_iter_simulate(self, node, iterations,
                                          step_profile=None):
        '''
        Simulate from the given node for the given number of iterations.
        
        (Internal function for history-dependent simulations only.)
        
        The results are implemented into the tree. Note that the crunching for
        this is done synchronously, i.e. in the currrent thread.
        
        This returns a generator that yields all the nodes one-by-one, from the
        initial node to the final one.
        
        A step profile may be passed to be used with the step function.
        '''
        
        if step_profile is None: step_profile = garlicsim.misc.StepProfile()
        
        path = node.make_containing_path()
        history_browser = \
            garlicsim.synchronous_crunching.HistoryBrowser(path, end_node=node)
        
        iterator = self.simpack_grokker.step_generator(history_browser,
                                                       step_profile)
        finite_iterator = cute_iter_tools.shorten(iterator, iterations)
        finite_iterator_with_lock = cute_iter_tools.iter_with(
            finite_iterator,
            self.tree.lock.write
        )
        
        current_node = node
        
        yield current_node
        
        try:
            for current_state in finite_iterator_with_lock:
                
                history_browser.end_node = current_node
                history_browser.path = current_node.make_containing_path()
                # Similarly to the `__history_dependent_simulate` method, here
                # we also need to recreate the path. But in this case we need to
                # do it not only on the first run, but on *each* run of the
                # loop, because this is a generator, and the user may wreak
                # havoc with the tree between `yield`s, causing our original
                # path not to lead to the end_node anymore.                
                # todo optimize: The fact we recreate a path every time might be
                # costly.
                
                current_node = self.tree.add_state(current_state,
                                                   parent=current_node,
                                                   step_profile=step_profile)
                    
                yield current_node
        
        except garlicsim.misc.WorldEnd:
            self.tree.make_end(current_node, step_profile)
                
    
    def __non_history_dependent_iter_simulate(self, node, iterations,
                                              step_profile=None):
        '''
        Simulate from the given node for the given number of iterations.
        
        (Internal function for non-history-dependent simulations only.)
        
        The results are implemented into the tree. Note that the crunching for
        this is done synchronously, i.e. in the currrent thread.
        
        This returns a generator that yields all the nodes one-by-one, from the
        initial node to the final one.
        
        A step profile may be passed to be used with the step function.
        '''
        
        if step_profile is None: step_profile = garlicsim.misc.StepProfile()

        state = node.state
                
        iterator = self.simpack_grokker.step_generator(state, step_profile)
        finite_iterator = cute_iter_tools.shorten(iterator, iterations)
        finite_iterator_with_lock = cute_iter_tools.iter_with(
            finite_iterator,
            self.tree.lock.write
        )
        
        current_node = node
        
        yield current_node
        
        try:
            for current_state in finite_iterator_with_lock:
                current_node = self.tree.add_state(current_state,
                                                   parent=current_node,
                                                   step_profile=step_profile)
                yield current_node
        
        except garlicsim.misc.WorldEnd:
            self.tree.make_end(current_node, step_profile)
            
    
    def __getstate__(self):
        my_dict = dict(self.__dict__)
        
        del my_dict['crunching_manager']
        del my_dict['simpack_grokker']
        
        return my_dict
    
    
    def __setstate__(self, pickled_project):
        self.__init__(pickled_project["simpack"])
        self.__dict__.update(pickled_project)
        
        
    def __repr__(self):
        '''
        Get a string representation of the project.
        
        Example output:
        <garlicsim.Project containing 101 nodes and employing 4 crunchers at
        0x1f668d0>
        '''
        # Todo: better have the simpack mentioned here, not doing it cause it's
        # currently in a module wrapper.
        
        nodes_count = len(self.tree.nodes)
        crunchers_count = len(self.crunching_manager.crunchers)
                                   
        return '''<%s containing %s nodes and employing %s crunchers at \
%s>''' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                   ),
                   nodes_count,
                   crunchers_count,
                   hex(id(self))
               )
        
        
        
        
        
        


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This package defines a variety of little objects that are completely decoupled
from garlicsim and do not belong anywhere else.
'''


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the SimpackGrokker class and the InvalidSimpack exception.

See their documentation for more details.
'''

import functools
import types
import imp

from garlicsim.general_misc import import_tools
import garlicsim.general_misc.caching

from garlicsim.misc import AutoClockGenerator, StepIterator, InvalidSimpack
from garlicsim.misc import simpack_tools
import garlicsim
from . import misc

from .settings import Settings


class SimpackGrokker(object):
    '''Encapsulates a simpack and gives useful information and tools.'''
    
    __metaclass__ = garlicsim.general_misc.caching.CachedType

    @staticmethod
    def create_from_state(state):
        simpack = simpack_tools.get_from_state(state)
        return SimpackGrokker(simpack)
    
    
    def __init__(self, simpack):
        self.simpack = simpack
        self.__init_analysis()
        self.__init_analysis_settings()

        
    def __init_analysis(self):
        '''Analyze the simpack.'''
        
        simpack = self.simpack
        
        try:
            State = simpack.State
        except AttributeError:
            raise InvalidSimpack('''The %s simpack does not define a `State` \
class.''' % simpack.__name__)
        
        if not issubclass(State, garlicsim.data_structures.State):
            raise InvalidSimpack('''The %s simpack defines a State class, but \
it's not a subclass of `garlicsim.data_structures.State`.''' % \
                                                             simpack.__name__)

        self.simple_non_history_step_defined = hasattr(State, "step")
        self.non_history_step_generator_defined = \
            hasattr(State, "step_generator")
        self.simple_history_step_defined = hasattr(simpack, "history_step")
        self.history_step_generator_defined = hasattr(simpack,
                                                      "history_step_generator")
        
        self.non_history_step_defined = \
            (self.simple_non_history_step_defined or \
             self.non_history_step_generator_defined)
        
        self.history_step_defined = (self.simple_history_step_defined or \
                                     self.history_step_generator_defined)
        
        self.simple_step_defined = (self.simple_non_history_step_defined or \
                                    self.simple_history_step_defined)
        
        self.step_generator_defined = \
            (self.non_history_step_generator_defined or \
             self.history_step_generator_defined)
        
        if self.history_step_defined and self.non_history_step_defined:
            raise InvalidSimpack('''The %s simpack is defining both a \
history-dependent step and a non-history-dependent step - which is forbidden.\
''' % simpack.__name__)
        
        if not (self.simple_step_defined or self.step_generator_defined):
            raise InvalidSimpack('''The %s simpack has not defined any kind \
of step function.''' % simpack.__name__)
        
        self.history_dependent = self.history_step_defined
        
    
    def __init_analysis_settings(self):
        '''Analyze the simpack to produce a Settings object.'''
        # todo: consider doing this in Settings.__init__
        
        # We want to access the `.settings` of our simpack, but we don't know if
        # our simpack is a module or some other kind of object. So if it's a
        # module, we'll `try` to import `settings`.
        
        self.settings = Settings()        
        
        if isinstance(self.simpack, types.ModuleType) and \
           not hasattr(self.simpack, 'settings'):
            
            # The `if` that we did here means: "If there's reason to suspect
            # that self.simpack.settings is a module that exists but hasn't been
            # imported yet."
            
            settings_module_name = ''.join((
                self.simpack.__name__,
                '.settings'
            ))
            
            import_tools.import_if_exists(settings_module_name)
            # This imports the `settings` submodule, if it exists, but it
            # does *not* keep a reference to it. We'll access `settings` as
            # an attribute of the simpack below.
            
        # Checking if there are original settings at all. If there aren't, we're
        # done.
        if hasattr(self.simpack, 'settings'):
            
            original_settings = getattr(self.simpack, 'settings')
        
            for (key, value) in vars(self.settings).items():
                if hasattr(original_settings, key):
                    actual_value = getattr(original_settings, key)
                    setattr(self.settings, key, actual_value)
            # todo: currently throws away unrecognized attributes from the
            # simpack's settings.
                
        
    def step(self, state_or_history_browser, step_profile):
        '''
        Perform a step of the simulation.
        
        The step profile will specify which parameters to pass to the simpack's
        step function.
        ''' 
        auto_clock_generator = AutoClockGenerator()
        if isinstance(state_or_history_browser,
                      garlicsim.data_structures.State):
            state = state_or_history_browser
        else:
            state = state_or_history_browser.get_last_state()
        auto_clock_generator.make_clock(state)

        if self.simple_step_defined:
            step_function = self.simpack.history_step if \
                          self.history_dependent else self.simpack.State.step
            result = step_function(state_or_history_browser,
                                   *step_profile.args,
                                   **step_profile.kwargs)
        else: # self.step_generator_defined is True
            step_generator = self.simpack.history_step_generator if \
                          self.history_dependent else \
                          self.simpack.State.step_generator
            iterator = step_generator(state_or_history_browser,
                                      *step_profile.args,
                                      **step_profile.kwargs)
            result = next(iterator)
            
        result.clock = auto_clock_generator.make_clock(result)
        return result
            
        
    
    def step_generator(self, state_or_history_browser, step_profile):
        '''
        Step generator for crunching states of the simulation.
        
        The step profile will specify which parameters to pass to the simpack's
        step function.
        '''
        
        if self.step_generator_defined:
            step_generator = self.simpack.history_step_generator if \
                           self.history_dependent else \
                           self.simpack.State.step_generator
            return StepIterator(state_or_history_browser, step_profile,
                                step_generator=step_generator)
        else:
            assert self.simple_step_defined
            simple_step = self.simpack.history_step if self.history_dependent \
                        else self.simpack.State.step
            return StepIterator(state_or_history_browser, step_profile,
                                simple_step=simple_step)
        
        
        



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines several miscellaneous objects.

These objects are important enough to be defined near the root of the
`garlicsim` package but not important enough to be put in the main
namespace.
'''

from .step_copy import StepCopy
from .exceptions import (InvalidSimpack, SimpackError, GarlicSimWarning,
                        GarlicSimException, WorldEnd)
from .auto_clock_generator import AutoClockGenerator
from .base_history_browser import BaseHistoryBrowser
from .step_iterator import StepIterator
from . import persistent
from .persistent import Persistent, CrossProcessPersistent
from .step_profile import StepProfile
from .nodes_added import NodesAdded
from .simpack_grokker import SimpackGrokker
from . import caching
from . import settings_constants
from . import simpack_tools



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the StepIterator class.

See its documentation for more information.
'''


import copy

import garlicsim
from garlicsim.misc import SimpackError, AutoClockGenerator


class StepIterator(object):
    '''
    An iterator that uses a simpack's step to produce states.
    
    The StepIterator uses under the hood the simpack's step function, be it a
    simple step function or a step generator. Using a StepIterator instead of
    using the simpack's step has a few advantages:
    
    1. The StepIterator automatically adds clock readings if the states are
       missing them.
    2. It's possible to change the step profile while iterating.    
    3. Unless the step function raises `WorldEnd` to end the simulation, this
       iterator is guaranteed to be infinite, even if the simpack's iterator is
       finite.
    
    And possibly more.  
    '''
    # todo: make stuff private here?
    def __init__(self, state_or_history_browser, step_profile,
                 simple_step = None, step_generator=None):
        '''
        Constructor.
        
        The iterator will use either a simple step function or a step generator
        under the hood. You have to supply either one or the other, but not
        both.
        '''

        assert [simple_step, step_generator].count(None) == 1
        
        self.simple_step = simple_step
        self.step_generator = step_generator
        self.raw_iterator = None
        
        self.step_profile = copy.deepcopy(step_profile)
        self.history_dependent = isinstance(state_or_history_browser,
                                            garlicsim.misc.BaseHistoryBrowser)
        if self.history_dependent:
            self.current_state = None
            self.history_browser = state_or_history_browser
        else:
            self.current_state = state_or_history_browser
            self.history_browser = None
            
        self.auto_clock_generator = AutoClockGenerator()
        if self.current_state:
            self.auto_clock_generator.make_clock(self.current_state)
            
        self.step_profile_changed = False
            
    def __iter__(self): return self
    
    def __next__(self):
        '''Crunch the next state.'''
        self.current_state = self.__get_new_state()
        self.auto_clock(self.current_state)
        return self.current_state
        
    def __get_new_state(self): # todo: rename?
        '''Internal method to crunch the next state.'''
        if self.simple_step:
            thing = self.history_browser if self.history_dependent else \
                  self.current_state
            return self.simple_step(thing,
                                    *self.step_profile.args,
                                    **self.step_profile.kwargs)
        else: # self.step_generator is not None
            self.rebuild_raw_iterator_if_necessary()
            try:
                return next(self.raw_iterator)
            except StopIteration:
                try:
                    self.rebuild_raw_iterator()
                    return next(self.raw_iterator)
                except StopIteration:
                    raise SimpackError('''Step generator's iterator raised
StopIteration before producing a single state.''')
            
                
    def rebuild_raw_iterator_if_necessary(self):
        '''
        Rebuild the internal iterator if necessary.
        
        This is relevant only when we're using a simpack's step generator and
        not its simple step.
        '''
        if (self.raw_iterator is None) or self.step_profile_changed:
            self.rebuild_raw_iterator()
            self.step_profile_changed = False
            
            
    def rebuild_raw_iterator(self):
        '''
        Rebuild the internal iterator.
        
        This is relevant only when we're using a simpack's step generator and
        not its simple step.
        '''
        thing = self.current_state or self.history_browser
        self.raw_iterator = self.step_generator(thing,
                                                *self.step_profile.args,
                                                **self.step_profile.kwargs)
                
        
    def auto_clock(self, state):
        '''If the state has no clock reading, give it one automatically.'''
        state.clock = self.auto_clock_generator.make_clock(state)
        
        
    def set_step_profile(self, step_profile):
        '''
        Set a new step profile for the StepIterator to use.

        The StepIterator will immediately adopt the new step profile, and any
        states that will be crunched from this point on will be crunched using
        the new step profile. (At least until it is changed again.)
        '''
        self.step_profile = copy.deepcopy(step_profile)
        self.step_profile_changed = True
        

    



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A module that defines the Path class and a few related exceptions.

See its documentation for more information.
'''

import copy as copy_module # Avoiding name clash.
import builtins

from garlicsim.general_misc import binary_search
from garlicsim.general_misc.math_tools import cmp
from garlicsim.general_misc import misc_tools
from garlicsim.general_misc import cute_iter_tools

from garlicsim.misc import GarlicSimException

from .node import Node
from .block import Block
# We are doing `from tree import Tree` in the bottom of the file.


__all__ = ['Path', 'PathError', 'PathOutOfRangeError', 'EndNotReached',
           'StartNotReached']


class PathError(GarlicSimException):
    '''Path-related exception.'''

class PathLookupError(PathError, LookupError):
    '''Path-related exception.'''    
    
class PathOutOfRangeError(PathError, IndexError):
    '''Nodes are requested from the path which are out of its range.'''

class EndNotReached(PathError):
    '''
    An end node/block is specified but it turns out not to be on the path.
    '''
    # todo: consider subclass from one of the obscure exceptions like
    # LookupError

class StartNotReached(PathError):
    '''
    A start node/block is specified but it turns out not to be on the path.
    '''
    

class Path(object):
    '''
    A path represents a line of nodes in a tree.
    
    A tree may be complex and contain many junctions, but a path is a direct
    line through it. Therefore, a path object contains information about which
    child to choose when going through a node which has multiple children.
    
    Some of Path's method accept `start` and `end` parameters for specifying a
    sub-range inside the path. It should be noted that this range will include
    both endpoints.
    '''
    
    def __init__(self, tree, root=None, decisions={}):
        '''
        Construct the path.
        
        `tree` is the tree that this path is on. `root` is the node from which
        the path begins. `decisions` is a dictionary of the form
        {node_which_forks: node_to_continue_to, ... }. It usually contains as
        keys only nodes that have more than one child.
        '''

        self.tree = tree
        '''The tree that this path is on.'''
        
        self.root = root
        '''The root node.'''
        
        self.decisions = dict(decisions)
        '''
        The decisions dict says which fork of the road the path chooses.
        It's of the form {node_which_forks: node_to_continue_to, ... }
        '''
         # todo: Use shallow copy instead of dict.__init__. Will allow
         # dictoids.

         
    def __len__(self, start=None, end=None):
        '''
        Get the length of the path in nodes.
        
        You can optionally specify `start` and/or `end`, which may be either
        nodes or blocks.
        '''
        if start is None and self.root is None:
            return 0
            
        return sum(len(thing) for thing in 
                   self.iterate_blockwise(start=start, end=end))


    def __iter__(self, start=None, end=None):
        '''
        Iterate over the nodes in the path.
        
        You can optionally specify `start` and/or `end`, which may be either
        nodes or blocks.
        '''
        if start is None:
            if self.root is None:
                raise StopIteration
            current = self.root
        else:
            current = start
        
        current = current if isinstance(current, Node) else current[0]
            
        while True:
            
            yield current
            
            if end is not None:
                if current is end:
                    raise StopIteration
                elif isinstance(end, Block) and (current in end):
                    if current.is_last_on_block():
                        raise StopIteration
                        
            try:
                current = self.next_node(current)           
            except PathOutOfRangeError:
                if end is not None:
                    raise EndNotReached
                raise StopIteration

            
    def __reversed__(self):
        '''Iterate on the nodes in the path from end to start.'''
        # todo: may add start and end
        current_node = self[-1]
        while current_node is not None:
            yield current_node
            current_node = current_node.parent
            
            
    def iterate_blockwise(self, start=None, end=None):
        '''
        Iterate on the path, yielding blocks when possible.
        
        You can optionally specify `start` and/or `end`, which may be either
        nodes or blocks.
        '''

        if start is None:
            if self.root is None:
                raise StopIteration
            current = self.root
        else: # start is not None
            current = start
            if isinstance(start, Node) and start.block is not None and \
               start.is_first_on_block() is False:
                # We are starting iteration on a node in a block. (And it's not
                # the first one on the block.) We will not yield its block at
                # all. We'll yield all the nodes one by one until the next
                # node/block in the tree.
                index_of_start = start.block.index(start)
                for current in start.block[index_of_start:]:
                    yield current
                    if current is end:
                        raise StopIteration
                    
                assert current is start.block[-1]
                
                try:
                    current = self.next_node(current)
                except PathOutOfRangeError:
                    if end is not None:
                        raise EndNotReached
                    raise StopIteration
                
        while True:
            if current.block is not None:
                if end is not None:
                    if end is current.block:
                        yield current.block
                        raise StopIteration
                    elif end in current.block:
                        index_of_end = current.block.index(end)
                        for thing in current.block[ 0 : (index_of_end + 1) ]:
                            yield thing
                        raise StopIteration
                    else: # end isn't here
                        current = current.block
                        yield current    
                else: # end is None
                    current = current.block
                    yield current
            else: # current.block is None
                yield current
                if current.is_overlapping(end):
                    raise StopIteration
            try:
                current = self.next_node(current)
            except PathOutOfRangeError:
                if end is not None:
                    raise EndNotReached
                raise StopIteration
    
            
    def iterate_blockwise_reversed(self, end, start=None):
        '''
        Iterate backwards on the path, yielding blocks when possible.
        
        You must specify an `end`. You may optionally specify a `start`. Both of
        these may be either nodes or blocks.
        '''
        current = end
        if isinstance(end, Node) and end.block is not None and \
           end.is_last_on_block() is False:
            # We are starting iteration on a node in a block. (And it's not the
            # last one on the block.) We will not yield its block at all. We'll
            # yield all the nodes one by one until the previous node/block in
            # the tree.
            index_of_end = end.block.index(end)
            for current in end.block[ index_of_end : : -1 ]:
                yield current
                if current is start:
                    raise StopIteration

            assert current is end.block[0]
            
            if isinstance(current, Node):
                current = current.parent
            else: # isinstance(current, Block)
                current = current[0].parent
            if current is None:
                if start is not None:
                    raise StartNotReached
                raise StopIteration
                
        while True:
            if current.block is not None:
                if start is not None:
                    if start is current.block:
                        yield current.block
                        raise StopIteration
                    elif start in current.block:
                        index_of_start = current.block.index(start)
                        for thing in current.block[ : (index_of_start - 1) : -1 ]:
                            yield thing
                        raise StopIteration
                else: # start is None
                    current = current.block
                    yield current
            else: # current.block is None
                yield current
                if current.is_overlapping(start):
                    raise StopIteration

            if isinstance(current, Node):
                current = current.parent
            else: # isinstance(current, Block)
                current = current[0].parent
            if current is None:
                if start is not None:
                    raise StartNotReached
                raise StopIteration
            

    def __contains__(self, thing, start=None, end=None):
        '''Return whether the path contains the specified node/block.'''
        
        assert isinstance(thing, Node) or isinstance(thing, Block)

        for candidate in self.iterate_blockwise(start=start, end=end):
            if candidate is thing:
                return True
            elif isinstance(candidate, Block) and thing in candidate:
                return True
            
        return False


    def next_node(self, thing):
        '''
        Return the node on the path which is next after `thing`.
        
        If we've come to a fork for which we have no key in the decisions dict,
        we choose the first child node in the parent node's `children`, and
        update the decisions dict to point to it as well.
        '''
        
        # We're dealing with the case of 1 child first, because it's the most
        # common.
        real_thing = thing if isinstance(thing, Node) else thing[-1]
        kids = real_thing.children
        if len(kids) == 1:
            return kids[0]
        
        if (thing in self.decisions) or (real_thing in self.decisions):
            return self.decisions.get(thing, None) or \
                   self.decisions.get(real_thing, None)
        
        if len(kids) > 1:
            kid = kids[0]
            # Whether it should take `kids[0]` or `kids[-1]` is a subject for
            # debate. The question is, when you update the tree, do you want the
            # old paths to point to the new nodes or the old?
            self.decisions[real_thing] = kid
            return kid

        else: # no kids
            raise PathOutOfRangeError

    

    def __getitem__(self, index, end=None):
        '''
        Get a node by its index number in the path.

        You can optionally specify an end node in which the path ends.
        '''
        #todo: allow slicing? make Path.states for this and for iterating?
        #todo: generalize `end` to blocks
        assert isinstance(index, int)
        
        
        if index >= 0:
            return self.__get_item_positive(index, end=end)
        else:
            return self.__get_item_negative(index, end=end)

        
    def __get_item_negative(self, index, end=None):
        '''
        Get a node by its index number in the path. Negative indices only.

        You can optionally specify an end node in which the path ends.
        '''
        if end is None:
            end = self.get_last_node()
        else:
            assert isinstance(end, Node) or isinstance(end, Block)
        if index == -1:
            return end
        
        my_index = -1
        
        if end.block:
            block = end.block
            index_of_end = block.index(end)
            
            my_index -= (index_of_end)
            
            if my_index <= index:
                return block[index - my_index]
            
            end = end.block[0]
        
        my_index += 1
            
        for thing in self.iterate_blockwise_reversed(end=end):
            my_index -= len(thing)
            if my_index <= index:
                if isinstance(thing, Block):
                    return thing[(index - my_index)]
                else:
                    assert my_index == index
                    return thing
                
        raise PathOutOfRangeError
        
    
    def __get_item_positive(self, index, end=None):
        '''
        Get a node by its index number in the path. Positive indices only.

        You can optionally specify an end node in which the path ends.
        '''
        # todo: supports blocks?
        my_index = -1
        answer = None
        for thing in self.iterate_blockwise(end=end):
            my_index += len(thing)
            if my_index >= index:
                if isinstance(thing, Block):
                    answer = thing[(index-my_index) - 1]
                    break 
                else:
                    assert my_index == index
                    answer = thing
                    break
        if answer is not None:
            return answer
        raise PathOutOfRangeError

    def get_last_node(self, start=None):
        '''
        Get the last node in the path.
        
        You can optionally specify `start`, which may be either a node or block.
        '''

        thing = None # Setting to None before loop, so we know if loop was empty
        
        for thing in self.iterate_blockwise(start=start):
            pass

        if isinstance(thing, Block):
            return thing[-1]
        elif isinstance(thing, Node):
            return thing
        else: # thing is None
            raise PathOutOfRangeError('''You asked for the last node in the \
path, but it's completely empty.''')
    
        
    def get_node_by_clock(self, clock, rounding=binary_search.CLOSEST,
                          end_node=None):
        '''
        Get a node according to its clock.
        
        See documentation of binary_search.roundings for details about rounding
        options.
        '''
        
        my_function = lambda node: node.state.clock
        return self.get_node_by_monotonic_function(function=my_function,
                                                   value=clock,
                                                   rounding=rounding,
                                                   end_node=end_node)    
        
    
    def get_node_by_monotonic_function(self, function, value,
                                       rounding=binary_search.CLOSEST,
                                       end_node=None):
        '''
        Get a node by specifying a measure function and a desired value.
        
        The function must be a monotonic rising function on the timeline.
        
        See documentation of binary_search.roundings for details about rounding
        options.
        '''
        
        assert issubclass(rounding, binary_search.Rounding)
        
        both = \
             self.__get_node_by_monotonic_function_with_both_rounding(function,
                                                                      value)
        if end_node is not None:
            new_both = list(both)
            end_clock = end_node.state.clock
            if new_both[0] and new_both[0].state.clock >= end_clock:
                new_both[0] = end_node
            if new_both[1] and new_both[1].state.clock >= end_clock:
                new_both[1] = None
            both = new_both
            
        binary_search_profile = \
            binary_search.BinarySearchProfile(self, function, value, both)
        
        return binary_search_profile.results[rounding]
                    
    
    def __get_node_by_monotonic_function_with_both_rounding(self, function,
                                                            value):
        '''
        Get a node by specifying a measure function and a desired value.
        
        The function must be a monotonic rising function on the timeline.
        
        The rounding option used is `binary_search.BOTH`.
        
        Note that this function does not let you specify an end node. Currently
        we're not optimizing for the case where you have an end node and this
        function might waste resources exploring beyond it.
        '''
        
        root = self.root
        
        cmp_root = cmp(function(root), value)
        
        if cmp_root == 1: # function(root) > value
            return (None, root)
        if cmp_root == 0: # function(root) == value
            return (root, root)

        assert cmp_root == -1 # and function(root) < value
        
        # Now we've established that the first node in the path has a strictly
        # lower value than what we're looking for.
        
        # A rule we will strictly obey in this function: `low` will always be a
        # member whose value is lower than the desired value. (Strictly lower,
        # meaning not lower-or-equal.)
        
        low = self.root
        
        for thing in self.iterate_blockwise():
            
            # Rule: Every time we inspect a new node/block, `low` will be the
            # node that is its immediate parent. i.e. The highest node possible
            # from those that we have previously examined.
            
            if isinstance(thing, Block):
                
                block = thing
                
                first = block[0]

                cmp_first = cmp(function(first), value)
                
                if cmp_first == -1: # function(first) < value
                    low = first
                
                elif cmp_first == 0: # function(first) == value
                    return (first, first)
                    
                else: # cmp_first == 1 and function(first) > value
                    return (low, first)
                    
                
                # At this point we know that the first node in the block has a
                # strictly lower value than the target value.
                
                last = block[-1]
                
                cmp_last = cmp(function(last), value)
                
                if cmp_last == -1: # function(last) < value
                    low = last
                    continue
                
                elif cmp_last == 0: # function(last) == value                
                    return (last, last)
                
                else: # cmp_last == 1 and function(last) > value
                    # The two final results are both in the block.
                    return binary_search.binary_search(
                        block, function, value, rounding=binary_search.BOTH
                    )
                    
                
            else: # thing is a Node
                
                node = thing
                
                cmp_node = cmp(function(node), value)
                                
                if cmp_node == -1: # function(node) < value
                    low = node
                    continue
                elif cmp_node == 0: # function(node) == value
                    return (node, node)
                else: # function(node) > value
                    return (low, node)
        

        # If the flow reached here, that means that even the last node in the
        # path has lower value than the value we're looking for.
        
        return (low, None)
            
    
    def get_node_occupying_timepoint(self, timepoint):
        '''
        Get the node which "occupies" the given timepoint.
        
        A node is considered to "occupy" a timepoint if it is the
        highest-clocked node before the timepoint, AND there exists another node
        which has a clock higher than timepoint (that higher node is not
        returned, it just has to exist for the first node to qualify as
        "occupying".)
        
        If no such node exists, returns None.
        '''
        return self.get_node_by_clock(timepoint,
                                      rounding=binary_search.LOW_IF_BOTH)
    

    def get_existing_time_segment(self, start_time, end_time):
        '''
        Get the existing time segment between `start_time` and `end_time`.
        
        Example: In the path the first node's clock reading is 3.2, the last is
        7.6.
        `start_time` is 2 and `end_time` is 5.
        The function will return [3.2, 5].
        '''

        clock_of_first = self.root.state.clock
        clock_of_last = self.get_last_node().state.clock
        
        if clock_of_first <= end_time and clock_of_last >= start_time:
            return [max(clock_of_first, start_time),
                    min(clock_of_last, end_time)]
        else:
            return None
    
        
    def modify_to_include_node(self, node):
        '''
        Modify the path to include the specified node.
        
        Optimization note: Don't try to check whether `node in path` before
        calling this method. It's more efficient to just call this method
        without checking first.
        '''
        new_path = node.make_past_path()
        self.root = new_path.root
        self.decisions.update(new_path.decisions)
    
    
    def states(self):
        '''Iterate over the states of the nodes in this path.'''
        # todo: Make fancier, like dict.keys in Py3. Probably create as object
        # in __init__
        for node in self:
            yield node.state
        
            
    def _get_lower_path(self, node):
        '''
        Get a lower path than this one.
        
        "Lower" means that in some point in the past the other path goes through
        a child node with a lower index number (in `children`) than this path.
        
        This method will return the highest path that is just below this path.
        '''
        return self._get_higher_path(node, reverse=True)
                
    
    def _get_higher_path(self, node, reverse=False):
        '''
        Get a higher path than this one.
        
        "Higher" means that in some point in the past the other path goes
        through a child node with a higher index number (in `children`) than
        this path.
        
        This method will return the lowest path that is just above this path.
        '''

        my_iter = builtins.reversed if reverse else iter
        
        wanted_clock = node.state.clock
        
        for (kid, parent) in cute_iter_tools.consecutive_pairs(reversed(self)):
            if len(parent.children) == 1:
                continue
            my_index = parent.children.index(kid)

            if reverse:
                if my_index > 0:
                    kids_to_try = parent.children[:my_index]
                    break
            
            if not reverse:
                if my_index < len(parent.children) -1:
                    kids_to_try = parent.children[my_index+1:]
                    break
        else:
            raise PathLookupError('This path is the %s one.' % \
                                  ('lowest' if reverse else 'highest'))
        
        for node in my_iter(kids_to_try):
            paths = node.all_possible_paths() # todo: make reversed argument
            for path in my_iter(paths):
                assert isinstance(path, Path)
                if path[-1].state.clock >= wanted_clock:
                    return path
        
        raise PathLookupError('''This path is the %s one which extends enough \
in the future to the clock of the specified node.''' % \
            ('lowest' if reverse else 'highest'))
               
            
    def __repr__(self):
        '''
        Get a string representation of the path.
        
        Example output:
        <garlicsim.data_structures.Path of length 43 at 0x1c822d0>
        '''
        return '<%s of length %s at %s>' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                   ),
                   len(self),
                   hex(id(self))
               )
    
    
    def copy(self):
        '''Make a shallow copy of the path.'''
        
        # Most of these things don't need duplicating, but just for
        # completeness' sake:
        tree = self.tree
        root = self.root
        decisions = self.decisions.copy()
        
        path = Path(tree=tree, root=root, decisions=decisions)
        
        return path
    
    __copy__ = copy
    
    def __eq__(self, other):
        # Currently horribly inefficient
        assert isinstance(other, Path)
        return list(self) == list(other)
    
    def __hash__(self):
        # Defining __hash__ because there's __eq__ which makes the default
        # __hash__ disappear on Python 3.
        return id(self)
        
    def __req__(self, other):
        return self.__eq__(other)
    
    
from .tree import Tree


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the NodesAdded class.

See its documentation for more information.
'''

class NodesAdded(int):
    '''
    sync_crunchers functions return a NodesAdded object instead of an int.
    
    NodesAdded is jsut a subclass of int which has a nice __repr__ saying
    '<7 nodes were added to the tree>' instead of just '7'.
    '''    
    # todo: this class is borderline redundant. Also, I think Maciej said not
    # to subclass builtin types in his pycon talk.
    def __repr__(self):
        return '<' + int.__repr__(self) + ' nodes were added to the tree>'
    
    def __add__(self, other):
        # If it's being added to another NodesAdded, keep it as a NodesAdded
        # object.
        int_result = int.__add__(self, other)
        if isinstance(other, NodesAdded):
            return NodesAdded(int_result)
        else:
            return int_result
        
    __radd__ = __add__
    
    def __sub__(self, other):
        # If it's being substracted with another NodesAdded, keep it as a
        # NodesAdded object.
        int_result = int.__sub__(self, other)
        if isinstance(other, NodesAdded):
            return NodesAdded(int_result)
        else:
            return int_result
        
    __rsub__ = __sub__
    
    def __pos__(self):
        return self
    
    def __neg__(self):
        return NodesAdded(int.__neg__(self))
            


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A module that defines the Block class and the related BlockError exception.

See the documentation of Block for more information.
'''

from garlicsim.general_misc import logic_tools
from garlicsim.general_misc import misc_tools

from garlicsim.misc import GarlicSimException

from .tree_member import TreeMember
# We are doing `from .node import Node` in the bottom of the file.

__all__ = ["Block", "BlockError"]


class BlockError(GarlicSimException):
    '''Block-related exception.'''

    
class Block(TreeMember):
    '''
    Succession of similar natural nodes in the tree.
    
    Blocks make the tree more organized and easy to browse, and improve
    performance.

    When you're doing a simulation, often you'll have a succession of 1000+
    natural nodes, which were created "organically", each from its parent,
    by simulation. Having a block to group these nodes together imporves
    efficiency.

    Who qualifies to get wrapped in a block? A succession of untouched nodes,
    which:
        1. Is at least 2 nodes in number.
        2. All members, except the last one, must have no ends, and no
           children except their successor in the block.
        3. The last node may have any kinds of children and ends.
        4. All members share the same step_profile.

    If you want to check whether a certain node is in a block or not,
    check its ".block" attribute.

    '''
    def __init__(self, node_list):
        '''Construct a block from the members of node_list.'''
        self.alive = True
        self.__node_list = []
        self.add_node_list(node_list)

    def soft_get_block(self):
        return self
    
    def append_node(self, node):
        '''
        Append a single node to the block.
        
        If the block's node list is empty, the node will be added
        unconditionally. If the node list contains some nodes, the new node
        must be either a child of the last node or the parent of the first one.
        '''
        
        assert self.alive
        
        if not self.__node_list:
            # If the node list is [], let's make it [node].
            self.__node_list.append(node)
            node.block = self
            return
        
        if node.step_profile != self.get_step_profile():
            raise BlockError('''Tried to add node which has a different
step_profile.''')
            
        
        # If the flow reached here, the block is not empty.
        last_in_block = self.__node_list[-1]
        if node.parent == last_in_block:
            # We're appending the node to the end of the block.
            self.__node_list.append(node)
            node.block = self
            return
        
        first_in_block = self.__node_list[0]
        if node == first_in_block.parent:
            # We're appending the node to the start of the block.
            self.__node_list.insert(0, node)
            node.block = self
            return
        
        raise BlockError('''Tried to add a node which is not a direct \
successor or a direct ancestor of the block.''')

    
    def add_node_list(self, node_list):
        '''
        Add a list of nodes to the block.
        
        These nodes must already be successive to each other.
        Also, one of the following conditions must be true:
            1. The first node in the list is a child of the last node in the
               block.
            2. The last node in the list is the parent of the first node in
               the block.
        '''
        
        assert self.alive
        
        if not node_list:
            return
        
        if len(node_list) == 1:
            self.append_node(node_list[0])
            return
        
        if not logic_tools.all_equal((node.step_profile for node
                                      in node_list)):
            raise BlockError('''Tried to add node list that doesn't share the \
same step options profile.''')
        
        sample_step_profile = node_list[0].step_profile
        
        if self.__node_list and \
           sample_step_profile != self.get_step_profile():
            raise BlockError('''Tried to add nodelist which contains node \
that has a different step_profile.''')
        
        # We now make sure the node_list is successive, untouched, and has no
        # unwanted children.
        for i in range(len(node_list)):
            if (i >= 1) and (node_list[i].parent != node_list[i-1]):
                raise BlockError('Tried to add non-consecutive nodes to block.')
            if (len(node_list) - i >= 2) and (len(node_list[i].children) != 1):
                raise BlockError('''Tried to add to the block a node which \
doesn't have exactly one child, and not as the last node in the block.''')
            if node_list[i].touched:
                raise BlockError("Tried to add touched nodes to block.")
        
        if not self.__node_list:
            # If the node list is empty, our job is simple.
            self.__node_list = list(node_list)
            for node in node_list:
                node.block = self
            return
        
        if node_list[0].parent == self.__node_list[-1]:
            self.__node_list = self.__node_list + node_list
        elif self.__node_list[0].parent == node_list[-1]:
            self.__node_list = node_list + self.__node_list
        else:
            raise BlockError('List of nodes is not adjacent to existing nodes.')

        for node in node_list:
            node.block = self

            
    def split(self, node):
        '''
        Split the block into two blocks.
        
        `node` would be the last node of the first block of the two. If either
        of the new blocks will contain just one node, that block will get
        deleted and the single node will become blockless.
        '''
        assert self.alive
        assert node in self
        i = self.__node_list.index(node)
        second_list = self.__node_list[i+1:]
        self.__node_list = self.__node_list[:i+1]
        if len(second_list) >= 2:
            Block(second_list)
        else:
            for node in second_list:
                node.block = None
        if len(self.__node_list) <= 1:
            self.delete()


    def delete(self):
        '''Delete the block, leaving all its nodes without a block.'''
        assert self.alive
        for node in self:
            node.block = None
        self.__node_list = []
        self.alive = False

        
    def __delitem__(self, i):
        '''
        Remove a node or a slice of nodes from the block.
        
        Nodes are specified by index number, whether you're removing a single
        node or a slice of them.
        
        When removing a single node, only an edge node can be removed.
        
        Can only remove an edge node.        
        '''
        # todo: allow removing nodes from middle
        # todo: change argument name `i` and seperate to two methods.
        # todo: allow specifying by nodes instead of numbers, both in slices
        # and in single.
        assert self.alive
        
        if isinstance(i, int):
            if (i == 0) or (i == -1) or \
               (i == len(self) - 1) or (i == -len(self)):
                self.__node_list[i].block = None
                return self.__node_list.__delitem__(i)
            elif (-len(self) < i < len(self) - 1):
                    raise BlockError('''Can't remove a node from the \
middle of a block''')
            else:
                raise IndexError('''Tried to remove a node by index, \
while the index was bigger than the block's length.''')
        
        elif isinstance(i, slice):
            if i.start < 0:
                i.start += len(self)
            if i.stop < 0:
                i.stop += len(self)
            
            assert 0 <= i.start <= i.stop < len(self)
            
            start_node, end_node = [self[index] for index in (i.start, i.stop)]
            
            self.split(end_node)

            if self.alive is False:                
                return
            
            if i.start >= 1:                
                self.split(start_node.parent)
                
            if start_node.block is not None:
                start_node.block.delete()
            
        else:
            raise NotImplementedError

    
    def __contains__(self, thing):
        '''Return whether `thing` is a node which this block contains.'''
        # The argument is called `thing` and not `node` because we want to let
        # people put a block in, and we'll just give them False. Saves them
        # checking themselves if what they got is a node.
        assert self.alive
        return isinstance(thing, Node) and thing.block is self

    
    def __iter__(self):
        '''Iterate over the nodes in the block.'''
        assert self.alive
        return self.__node_list.__iter__()

    
    def __len__(self):
        '''Return the number of nodes in the block.'''
        assert self.alive
        return len(self.__node_list)

    
    def __getitem__(self, index):
        '''Get a node by index number from the block'''
        assert self.alive
        return self.__node_list.__getitem__(index)
    
    
    # def __getslice__(self, *args, **kwargs): #todo: can drop because of getitem?
    #     return self.__node_list.__getslice__(*args, **kwargs)

    
    def index(self, node):
        '''Get the index number of the specified node in the block.'''
        assert self.alive
        return self.__node_list.index(node)
    
    
    def get_step_profile(self):
        '''
        Get the step profile of the nodes in this block.
        
        The same profile is used in all of the nodes in the block.
        '''
        assert self.alive
        return self.__node_list[0].step_profile
     
    def is_overlapping(self, tree_member):
        '''
        Return whether this block overlaps with the given tree member.
        
        `tree_member` may be a block, in which case overlapping means being the
        same block. `tree_member` can also be a node, in which case overlapping
        means the node is contained in this block.
        '''
        assert self.alive
        
        if tree_member is None: return False
        if isinstance(tree_member, Block):
            return (self is tree_member)
        else:
            assert isinstance(tree_member, Node)
            return (self in tree_member)
    
    
    def make_containing_path(self):
        '''
        Create a path that contains this block.
        
        There may be multiple different paths that contain this block. This will
        return the one which points to the newest possible forks.
        
        Returns the path.
        '''
        return self[0].make_containing_path()
        
    
    
    def all_possible_paths(self):
        '''
        Get a list of all possible paths that contain this block.
        
        Note: There may be paths that contain this node which will not be
        identical to one of the paths given here, because these other paths may
        specify decisions that are not even on the same root as these paths.
        '''
        return self[0].all_possible_paths()
    
    
    
    def make_past_path(self):
        '''
        Create a path that contains this block.
        
        There may be multiple different paths that contain this node. This will
        return a path that doesn't specify any decisions after this node.
        '''
        return self[0].make_past_path()


    
    def get_all_leaves(self, max_nodes_distance=None, max_clock_distance=None):
        '''
        Get all leaves that are descendents of this block.
        
        Only leaves with a distance of at most `max_nodes_distance` in nodes or
        `max_clock_distance` in clock are returned. (Note this is an OR relation
        between the two condintions)
        
        Returns a dict of the form:
        
        {
            leaf1: {
                'nodes_distance': nodes_distance1,
                'clock_distance': clock_distance1,
            },            
            leaf2: {
                'nodes_distance': nodes_distance2,
                'clock_distance': clock_distance2,
            },
            # ...
        }
            
        '''
        return self[-1].make_containing_path(max_nodes_distance,
                                             max_clock_distance)

    
    
    def get_ancestor(self, generations=1, round=False):
        '''
        Get an ancestor of this block.
        
        `generations` specifies the number of generation that the returned
        ancestor should be above the current block. `round` determines how this
        method will behave if it was asked for too many generations back, and
        not enough existed. If `round` is True, it will return the root. If
        `round` is False, it will raise a LookupError.
        '''
        return self[0].get_ancestor(generations, round)

    
    
    def get_root(self):
        '''
        Get the root of this block.
        
        This means the node which is the parent of the parent of the parent
        of... the parent of this block.
        '''
        return self[0].get_root()

    
    def __repr__(self):
        '''
        Get a string representation of the block.
        
        Example output:
        <garlicsim.data_structures.Block of length 40 crunched with
        StepProfile(t=0.1) at 0x1c84d70>
        '''
        assert self.alive # todo: say "Dead block"
        return '<%s of length %s, crunched with %s at %s>' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                   ),
                   len(self),
                   self.get_step_profile(),
                   hex(id(self))
               )
        
from .node import Node


import time
import garlicsim
from garlicsim.bundled.simulation_packages import life
import pickle


state = life.make_random_state(40, 15)
print(state)
new_state = garlicsim.simulate(life, state)
p = garlicsim.Project(life)
n = p.root_this_state(state)
p.ensure_buffer(n, 100)
print((p.sync_crunchers()))
time.sleep(3)
print((p.sync_crunchers()))



print((pickle.dumps(p)))



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines functions for conducting simulations with synchronous crunching.

This means that the crunching is done in the main thread, without recruiting any
worker threads or worker processes.
'''

from .simulate import simulate
from .list_simulate import list_simulate
from .iter_simulate import iter_simulate
from .history_browser import HistoryBrowser

__all__ = ['simulate', 'list_simulate', 'list_simulate', 'HistoryBrowser']


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines the End class.

See its documentation for more information.
'''

from garlicsim.general_misc import misc_tools

from .tree_member import TreeMember
from .node import Node

    
class End(TreeMember):
    '''
    An end of the simulation.
    
    An `End` signifies that the simulation has ended. This is relevant in only
    some simpacks; Some simpacks have a concept of ending the simulation, and
    some don't. When a simulation was crunched and reached its end on some
    timeline, the last node on that timeline will have an `End` object added to
    its `.ends` list.
    '''
    
    def __init__(self, tree, parent, step_profile=None):
        
        self.tree = tree
        '''The tree in which this end resides.'''
        
        assert isinstance(parent, Node)
        self.parent = parent
        '''
        The parent node of this end.
        
        Note that this parent node will not have this end as a child; It will
        list the end in its `.ends` attribute.
        '''
        
        self.parent.ends.append(self)
        
        self.step_profile = step_profile
        '''The step options profile with which the end was reached.'''
        
        
    def __len__(self):
        '''Just return 1. This is useful because of blocks.'''
        return 1

    
        
    def soft_get_block(self):
        '''
        Just return `self`.
        
        (This is a method of all TreeMembers that returns the block that the
        tree member belongs to, if there is one. But an end never belongs to a
        block.)
        '''
        return self

    
    def make_containing_path(self):
        '''
        Create a path that leads to this end.
        
        Returns the path.
        '''
        return self.parent.make_containing_path()
    

    def all_possible_paths(self):
        '''
        Get a list of all possible paths that lead to this end.
        
        (This method was invented for nodes and blocks and makes sense for them;
        For an end, it will just return the one single path that leads to it,
        since there can't be any forks after an end.)
        
        Note: There may be paths that contain this node which will not be
        identical to one of the paths given here, because these other paths
        may specify decisions that are not even on the same root as these
        paths.
        '''
        return self.parent.all_possible_paths()
    
    
    def make_past_path(self):
        '''
        Create a path that leads to this end.
        
        (This method was invented for nodes and blocks and makes sense for them;
        For an end, the "past path" is identical to the one made by
        `make_containing_path`, since there can't be any forks after an end.)
        
        Returns the path.
        '''
        return self.parent.make_past_path()

    
    def get_all_leaves(self, max_nodes_distance=None, max_clock_distance=None):
        '''
        Get `{}`.
        
        (This method was invented for nodes and blocks and makes sense for them;
        There are no leaves, or anything else for that matter, that come after
        an end.)
        '''
        
        return {}

    
    def get_ancestor(self, generations=1, round=False):
        '''
        Get an ancestor of this end.
        
        `generations` specifies the number of generation that the returned
        ancestor should be above the current end. `round` determines how this
        method will behave if it was asked for too many generations back, and
        not enough existed. If `round` is True, it will return the root. If
        `round` is False, it will raise a LookupError.
        '''

        assert generations >= 0
        if generations == 0:
            return self
        if generations >= 1:
            return self.parent(generations - 1, round)

            
    def get_root(self):
        '''
        Get the root of this end.
        
        This means the node which is the parent of the parent of the parent
        of... the parent of this end.
        '''
        return self.parent.get_root()
    
    
    
    def is_overlapping(self, other):
        '''
        Return whether this end overlaps with the given entity.
        
        Returns True only if `other` is the same end.
        '''
        return self is other
    
    
    def __repr__(self):
        '''
        Get a string representation of the end.
        
        Example output:        
        <garlicsim.data_structures.End from state with clock 6.5, crunched with
        StepProfile(t=0.1), at 0x1ffde70>
        '''
        
        return '<%s from state with clock %s, crunched with %s, at %s>' % \
            (
                misc_tools.shorten_class_address(
                    self.__class__.__module__,
                    self.__class__.__name__
                    ),
                self.parent.state.clock,
                self.step_profile,
                hex(id(self))
            )
        

from .block import Block




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Personality class.

See its documentation for more information.
'''

import colorsys
from .persistent import Persistent

class Personality(object):
    '''
    A bunch of easy-to-remember attributes associated with a persistent object.
    
    These attributes include:
      * A human name
      * A light color
      * A dark color
    
    Each persistent object has a personality associated with it, generated
    automatically from its uuid. The personality makes it easy for humans to
    identify the persistent object.
    
    Colors are specified in RGB.
    '''
    
    def __init__(self, persistent):

        assert isinstance(persistent, Persistent)
        
        from . import human_names

        color_resolution = 100
        
        u = int(persistent._CrossProcessPersistent__uuid)
        
        (u, human_name_seed) = divmod(u, 5494)
        self.human_name = human_names.name_list[human_name_seed]
        
        color_seeds = []
        for i in range(4):
            (u, new_color_seed) = divmod(u, color_resolution)
            color_seeds.append(new_color_seed)
        

        normalized_color_seeds = \
            [color_seed * (1.0/color_resolution) for color_seed in color_seeds]
        
        light_color_hls = (normalized_color_seeds[0], 0.9, normalized_color_seeds[1])
        dark_color_hls = (normalized_color_seeds[2], 0.3, normalized_color_seeds[3])
        
        self.light_color = colorsys.hls_to_rgb(*light_color_hls)
        self.dark_color = colorsys.hls_to_rgb(*dark_color_hls)
        
                          
                        
                          



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the `simulate` function.

See its documentation for more info.
'''

import copy
import warnings

from garlicsim.general_misc import cute_iter_tools

import garlicsim
import garlicsim.misc
from . import history_browser as history_browser_module # Avoiding name clash

__all__ = ["simulate"]


def simulate(state, iterations=1, *args, **kwargs):
    '''
    Simulate from the given state for the given number of iterations.

    Any extraneous parameters will be passed to the step function.
    
    Returns the final state of the simulation.
    '''
    simpack_grokker = \
        garlicsim.misc.SimpackGrokker.create_from_state(state)
    step_profile = garlicsim.misc.StepProfile(*args, **kwargs)

    if not hasattr(state, 'clock'):
        state = copy.deepcopy(
            state,
            garlicsim.misc.persistent.DontCopyPersistent()
        )
        state.clock = 0
    
    if simpack_grokker.history_dependent:
        return __history_simulate(simpack_grokker, state, iterations,
                                  step_profile)
    else: # It's a non-history-dependent simpack
        return __non_history_simulate(simpack_grokker, state, iterations,
                                      step_profile)

    
def __history_simulate(simpack_grokker, state, iterations=1, step_profile=None):
    '''    
    Simulate from the given state for the given number of iterations.
    
    (Internal function, for history-dependent simulations only)

    A simpack grokker must be passed as the first parameter. A step profile may
    be passed to be used with the step function.
    
    Returns the final state of the simulation.
    '''
    if step_profile is None: step_profile = garlicsim.misc.StepProfile()
    tree = garlicsim.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    history_browser = history_browser_module.HistoryBrowser(path)
    
    iterator = simpack_grokker.step_generator(history_browser, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    current_state = current_node.state
    
    try:
        for current_state in finite_iterator:
            current_node = tree.add_state(current_state, parent=current_node)
    except garlicsim.misc.WorldEnd:
        pass
        
    final_state = current_state
    # Which is still here as the last value from the for loop
    
    return final_state


def __non_history_simulate(simpack_grokker, state, iterations=1,
                           step_profile=None):
    '''
    Simulate from the given state for the given number of iterations.
    
    (Internal function, for non-history-dependent simulations only.)

    A simpack grokker must be passed as the first parameter. A step profile may
    be passed to be used with the step function.
    
    Returns the final state of the simulation.
    '''
    if step_profile is None: step_profile = garlicsim.misc.StepProfile()
    iterator = simpack_grokker.step_generator(state, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    current_state = state
    
    try:
        for current_state in finite_iterator:
            pass
    except garlicsim.misc.WorldEnd:
        pass    
    
    final_state = current_state
    # Which is still here as the last value from the for loop
    
    return final_state


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Settings class.

See its documentation for more info.
'''

from . import misc

class Settings(object):
    '''A set of settings for a simpack.'''
    # todo: subclass from a pretty vars-shower
    
    def __init__(self):

        self.FORCE_CRUNCHER = None
        '''
        A cruncher that this simpack insists on using.
        
        This is useful because some simpacks can't be used with certain kinds of
        crunchers.
        '''
        # todo: allow both list and single
        
        self.DETERMINISM_FUNCTION = misc.default_determinism_function
        '''
        Function that takes a step profile and says whether it's deterministic.
        
        What this function says is, "If you do a simulation using this step
        profile, then you will have a deterministic simulation." (Or
        undeterministic, depends on the step profile.)
        
        This is useful because it allows garlicsim to detect if a simulation has
        reached a repititive state, so it can stop the crunching right there and
        avoid wasting resources.

        Note that this function does not return True or False: It returns a
        `DeterminismSetting` class. For details about those, see documentation
        in garlicsim.misc.settings_constants.settings.
        
        The function will return None if it's unknown whether the step profile
        is deterministic.
        '''

        self.SCALAR_STATE_FUNCTIONS = []
        '''
        List of scalar state functions given by the simpack.
        
        A scalar state function is a function from a state to a real number.
        These should be decorated by garlicsim.misc.cached.state_cache.
        '''
        
        self.SCALAR_HISTORY_FUNCTIONS = []
        '''
        List of scalar history functions given by the simpack.
        
        A scalar history function is a function from a history browser to a real
        number. These should be decorated by
        garlicsim.misc.cached.history_cache.
        '''


import time

import garlicsim
from garlicsim_wx.simulation_packages import life

if __name__ == "__main__":
        
    state = life.make_random_state()
    
    project = garlicsim.Project(life)
    
    node = project.root_this_state(state)
    
    project.ensure_buffer(node, 100)
    
    print((project.sync_crunchers()))
    
    time.sleep(5)
    
    print((project.sync_crunchers()))


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the AutoClockGenerator class.

See its documentation for more info.
'''

from garlicsim.general_misc.third_party.decorator import decorator

__all__ = ['AutoClockGenerator']

@decorator
def store(method, *args, **kwargs):
    '''Decorator for saving current state's clock for next autoclocking.'''
    self = args[0]
    result = method(*args, **kwargs)
    self.last_state_clock = result
    return result
    
class AutoClockGenerator(object):
    '''Device for creating clock readings for states that don't have them.'''
    def __init__(self):
        self.last_state_clock = None
    
    @store
    def make_clock(self, state):
        '''
        Obtain a clock reading for given state.
        
        If the state already has one, return it; If not, return the clock
        reading of the last state plus one; If this is the first state, return
        0.
        
        Important: The new clock is not added to the state. The state is not
        modified at all. The clock reading is merely returned and the user may
        attach it to the state himself.
        '''
        if hasattr(state, 'clock'):
            return state.clock
        else:
            if self.last_state_clock is not None: # can be 0
                return self.last_state_clock + 1
            else:       
                return 0
            
            


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines exceptions.'''

from .third_party import abc


class CuteException(Exception):
    '''
    Exception that uses its first line of documentation in lieu of a message.
    '''

    __metaclass__ = abc.ABCMeta


    def __init__(self, msg=None):
        # We use `None` as the default for `msg`, so the user can input '' to
        # force an empty message.
        
        if type(self) is CuteException:
            raise Exception('''You tried to create a CuteException instance, \
but the CuteException class is meant to be used as a base class for \
exceptions, and it can't be raised by itself.''')
        
        if msg is None:
            if self.__doc__:
                msg = self.__doc__.strip().split('\n')[0] 
                # Getting the first line of the documentation
            else:
                msg = ''
                
        Exception.__init__(self, msg)

        


import pickle

'''
class Fuga(object):
    def __init__(
'''
    
    
class Object(object):
    pass

def naturally_picklable(thing):
    try:
        pickle.dumps(thing)
        return True
    except Exception:
        return False

def prepare_for_pickling(thing):
    if naturally_picklable(thing):
        return thing
    
    old_dict = thing.__dict__

    if hasattr(thing, "__dict__"):
        old_dict = dict(thing.__dict__)
    else:
        old_list = [[name, getattr(thing, name)] for name in dir(thing)]
        old_dict = {}
        for key, value in old_list:
            old_dict[key] = value
        
    my_object = Object()
    new_dict = my_object.__dict__ = {}
    
    for key, value in list(old_dict.items()):
        if naturally_picklable(value):
            new_dict[key] = value
        else:
            new_dict[key] = prepare_for_pickling(value)
    
    return my_object
            

def preepy(module):
    
    old_dict = dict(module.__dict__)
        
    my_object = Object()
    new_dict = my_object.__dict__ = old_dict
    
    return my_object

def fuga(module):
    return (lambda x: x, (preepy(module),))

def dumps(thing):
    return pickle.dumps(prepare_for_pickling(thing))
        
import copyreg
module_type = type(copy_reg)
copyreg.pickle(module_type, fuga)
pickle.dumps(copy_reg)



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines the EndMarker class.

See its documentation for more info.
'''


class EndMarker(object):
    '''
    A marker used by crunchers to say that the simulation reached its end.
    
    This is used only in endable simulations. When the step function raises a
    WorldEnd exception, signifying that the simulation has ended, the cruncher
    will place an EndMarker in the work queue. (Where otherwise states will be
    placed.)
    
    The CrunchingManager will recognize the EndMarker and put an End to the
    timeline.
    '''